/*
 * $Source: /u/pfratar/src/files-2.1/src/RCS/configure.c,v $
 *
 * $Id: configure.c,v 6.13 1993/03/30 22:12:20 pfratar v2_1pl10 $
 *
 * files - full screen file manager for UNIX
 * Copyright (C) 1993 Paul Frattaroli
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
#define EXTERN
#include "global.h"
#undef EXTERN

#ifndef lint
static char *rcsid = "$Id: configure.c,v 6.13 1993/03/30 22:12:20 pfratar v2_1pl10 $";
#endif

#define NUM_CONF_PARMS  6
#define MIN_CONF_Y      2
#define CONF_X          5
#define CONF_VAL_X     30
#define CONF_DESC_Y    15

conf_t config_table[NUM_CONF_PARMS] = {
    { "Confirm on [q]uit:",
      "Ask you to confirm when you quit?",
      TRUE, MIN_CONF_Y, CONF_X, CONF_DESC_Y, CONF_X },
    { "Recursive [d]elete:",
      "Recursively delete directories, even if not empty?",
      FALSE, MIN_CONF_Y + 1, CONF_X, CONF_DESC_Y, CONF_X },
    { "Restricted mode:",
      "Restrict \"files\" to your home directory and its subdirectories?",
      FALSE, MIN_CONF_Y + 2, CONF_X, CONF_DESC_Y, CONF_X },
    { "Show hidden files:",
      "Show files with names that start with '.'?",
      TRUE, MIN_CONF_Y + 2, CONF_X, CONF_DESC_Y, CONF_X },
    { "Beep:",
      "Beep when an incorrect key is pressed?",
      TRUE, MIN_CONF_Y + 3, CONF_X, CONF_DESC_Y, CONF_X },
    { "Interactive Destination:",
      "Full screen selection of destination on [c]opy, [r]ename etc?",
      FALSE, MIN_CONF_Y + 4, CONF_X, CONF_DESC_Y, CONF_X }
/*
    { "Confirm on [d]elete:",
      "Ask you to confirm when you delete files/directories?",
      TRUE, MIN_CONF_Y + 4, CONF_X, CONF_DESC_Y, CONF_X },
*/
};

/*
 * configure_files() - Configure various options on the program.
 */
int
configure_files(void)
{
    int i, ch, ccurpos = MIN_CONF_Y;
    char cline[1024];

    clear_cwin();
    cwin_yxaddstr(0, 0,
        "Use cursor keys to move between options, space to toggle");
    cwin_yxaddstr(1, 0, "and enter/return to accept");

    clear_fwin();
    for( i = 0; i < NUM_CONF_PARMS; i++ ) {
        fwin_yxaddstr(ccurpos, CONF_X, config_table[i].ct_name);
        sprintf(cline, "%s", config_table[i].ct_val ? "Yes" : "No");
        fwin_yxaddstr(ccurpos, CONF_VAL_X, cline);
        ccurpos++;
    }
    ccurpos = MIN_CONF_Y;
    fwin_yxaddstr(CONF_DESC_Y, CONF_X,
            config_table[ccurpos - MIN_CONF_Y].ct_desc);
    wclrtoeol(fwin);
    wmove(fwin, ccurpos, CONF_VAL_X);
    wrefresh(fwin);

    do {
        ch = my_getch(fwin);
        if(isupper(ch) && ch < 256)
            ch = tolower(ch);
        switch(ch) {
            case CSR_DN:
            case CSR_RHT:
            case 'l':
            case 'j':
                ccurpos += (ccurpos == MIN_CONF_Y + NUM_CONF_PARMS - 1) ? 0 : 1;
                break;
            case CSR_UP:
            case CSR_LFT:
            case 'h':
            case 'k':
                ccurpos -= (ccurpos == MIN_CONF_Y) ? 0 : 1;
                break;
            case ' ':
                config_table[ccurpos - MIN_CONF_Y].ct_val =
                    config_table[ccurpos - MIN_CONF_Y].ct_val ? FALSE : TRUE;
                sprintf(cline, "%s",
                    config_table[ccurpos - MIN_CONF_Y].ct_val ? "Yes" : "No");
                fwin_yxaddstr(ccurpos, CONF_VAL_X, cline);
                wclrtoeol(fwin);
                break;
            default:
                break;
        }
        fwin_yxaddstr(CONF_DESC_Y, CONF_X,
                config_table[ccurpos - MIN_CONF_Y].ct_desc);
        wclrtoeol(fwin);
        wmove(fwin, ccurpos, CONF_VAL_X);
        wrefresh(fwin);
    } while(ch != '\n' && ch != '\r');

    write_configuration();

    /*
     * Changing some config parameters will change the way we want
     * to view directory contents.  So, dump all the info we have.
     * Unfortunate but necessary.
     */
    free_all_info();

    new_dir(".", curfilter);
    return(RCNOERR);
}

read_configuration()
{
    int cfd;
    char cfpath[MAXPATHLEN];
    struct stat tmpst;

    sprintf(cfpath, FMT_START_PATH, homedir, FILESDIR, CONFIGFILE);
    if((cfd = open(cfpath, O_RDONLY)) != -1) {
        stat(cfpath, &tmpst);
        if(tmpst.st_mtime >= progstat.st_mtime) {
            if(read(cfd, (char *)(&config_table[0]),
                    NUM_CONF_PARMS * sizeof(conf_t)) !=
                    NUM_CONF_PARMS * sizeof(conf_t)) {
                sprintf(cfpath, "%s: error in configuration file", progname);
                perror(cfpath);
            }
        } else {
#ifdef MIPS
            time_t ct[2];
#endif
            puts("Your configuration save file seems to be out of date.");
            printf("It will not work with this version of '%s'.\n", progname);
            printf("Please reconfigure '%s' using the [c]onfigure menu item.\n",
                   progname);
#ifdef MIPS
            ct[0] = ct[1] = time((long *)0);
            utime(cfpath, ct);
#else
            utime(cfpath, (struct utimbuf *)NULL);
#endif
            waitkey();
        }
        close(cfd);
    }
}

write_configuration()
{
    int cfd;
    char cfpath[MAXPATHLEN];

    sprintf(cfpath, FMT_START_PATH, homedir, FILESDIR, CONFIGFILE);
    if((cfd = open(cfpath, O_CREAT|O_WRONLY, 0644)) != -1) {
        if(write(cfd, (char *)(&config_table[0]),
                NUM_CONF_PARMS * sizeof(conf_t)) !=
                NUM_CONF_PARMS * sizeof(conf_t)) {
            sprintf(cfpath, "%s: error writing configuration file", progname);
            perror(cfpath);
        }
        close(cfd);
    }
}

