/*
 * defs.h - contains globally utilized #defines
 *
 * $Source: /u/pfratar/src/files-2.1/src/RCS/defs.h,v $
 *
 * files - full screen file manager for UNIX
 * Copyright (C) 1993 Paul Frattaroli
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "config.h"

#ifndef __Defs_h__
#define __Defs_h__

/*
 * Size of a file list string
 */
#define FLSIZE       1024

/*
 * Size of the default dircont array.
 * NUMENTRIES is the default size of the dircont array when it is first
 *   allocated.  If more space is required (ie. there are more entries in
 *   the directory than NUMENTRIES) an additonal BLOCKSIZE is realloc'd.
 *   This reallocation takes place until there is a large enought array to
 *   hold all the directory entries.
 */
#define NUMENTRIES 256
#define BLOCKSIZE 256

/*
 * Number of times to try and allocate memory.  If malloc fails, the
 * Alloc() function will retry allocating the memory it requires.
 */
#define MEMTRIES 6

/*
 * Various format strings and defines for the screen display.
 */
#define FLFORMAT " %10s  %10d  %14s"
#define TMFORMAT "%02d/%02d/%02d %02d:%02d"
#ifdef BOLDING
#define HDFORMAT " \\H%10s\\h ------ \\H%4s\\h -- \\H%4s\\h - \\H%4s\\h --"
#else
#define HDFORMAT " %10s ------ %4s -- %4s - %4s --"
#endif
#define FINFOFMT "%s\nOWNER: %s   GROUP: %s\nINODE: %lu   LINKS: %d   ATIME: %s   CTIME: %s"

#ifdef _Display_C_
/*
 * The menu strings
 */
#ifdef BOLDING
  /* First line for each of the respective 3 menus */
char *cwin_ml1[] = {
    "[\\Ha\\h]ttrib        [\\Hc\\h]opy      [\\Hd\\h]elete   [\\He\\h]dit   [\\Hh\\h]elp     [\\Hi\\h]nfo   [\\Hm\\h]enu",
    "[\\Hb\\h]ackup    [\\Hc\\h]onfigure   [\\Hd\\h]efine   [\\Hg\\h]rep   [\\Hh\\h]elp   [\\Hm\\h]enu   [\\Hq\\h]uit",
    "MUL: [\\Ha\\h]ttrib   [\\Hc\\h]opy   [\\Hd\\h]elete  [\\He\\h]dit    [\\Hh\\h]elp      [\\Hi\\h]nclude/exclude",
    "[\\Ha\\h]bort   [\\Hd\\h]one   [\\Hh\\h]elp   [\\Hn\\h]ame" };
  /* Second line for each of the respective 3 menus */
char *cwin_ml2[] = {
    "[\\Hn\\h]ew file/dir  c[\\Ho\\h]mmand   [\\Hp\\h]rint    [\\Hq\\h]uit   [\\Hr\\h]ename   [\\Hs\\h]ort   [\\Hy\\h]our cmd",
    "[\\Hv\\h]ersion",
    "MUL: c[\\Ho\\h]mmand  [\\Hp\\h]rint  [\\Hq\\h]uit    [\\Hr\\h]ename  [\\Hy\\h]our cmd",
    "Move to desired destination directory using normal navigation keys" };
#else
char *cwin_ml1[] = {
    "[a]ttrib        [c]opy      [d]elete   [e]dit   [h]elp     [i]nfo   [m]enu",
    "[b]ackup   [c]onfigure   [d]efine   [f]ilter   [g]rep   [h]elp   [m]enu",
    "MUL: [a]ttrib   [c]opy   [d]elete  [e]dit    [h]elp      [i]nclude/exclude",
    "[a]bort   [d]one   [h]elp   [n]ame" };
  /* Second line for each of the respective 3 menus */
char *cwin_ml2[] = {
    "[n]ew file/dir  c[o]mmand   [p]rint    [q]uit   [r]ename   [s]ort   [y]our cmd",
    "[q]uit     [v]ersion",
    "MUL: c[o]mmand  [p]rint  [q]uit    [r]ename  [y]our cmd",
    "Move to desired destination directory using normal navigation keys" };
#endif

/*
 * Keywin lines
 */
char *kwl1;
#ifdef BOLDING
char *kwl2 = " \\H^B\\h Back    \\H^D\\h Down  \\H^E\\h End   \\H^F\\h Fwd    \\H^I\\h Include  \\H^L\\h Redraw    \\H^T\\h Top  \\H^U\\h Up";
char *kwl3 = "  \\Hu\\h Parent  \\H^V\\h Chdir  \\H!\\h Shell  \\HUPCUR\\h Up  \\HDNCUR\\h Dn    \\HLCUR\\h Parent  \\HRCUR\\h Select";
#else
char *kwl2 = " ^B Back    ^D Down  ^E End   ^F Fwd    ^I Include  ^L Redraw    ^T Top  ^U Up";
char *kwl3 = "  u Parent  ^V Chdir  ! Shell  UPCUR Up  DNCUR Dn    LCUR Parent  RCUR Select";
#endif
#endif /* ifdef _Display_C_ */

/*
 * Widths of various fields needed to adjust display on the fly.
 */
#define ATTRIBWIDTH   10
#define DATETIMEWIDTH 14
#define SIZEWIDTH     10
#define SPACEWIDTH    10
#define NAME_HD_SIZE   5
#define MIN_SCR_WDTH  80
#define MIN_SCR_HGHT  18
#if 0
#define MIN_SCR_WDTH  ATTRIBWIDTH + DATETIMEWIDTH + SIZEWIDTH + SPACEWIDTH + NAME_HD_SIZE
#endif

/*
 * Number of lines in the command window.
 */
#define CWINSIZE 4
#define KEYWINSIZE 3

/*
 * Control cursor movement
 */
#define NOWARP    0
#define TOTOP     1
#define TOEND     2
#define PG_UP   256
#define PG_DN   257
#define HOME    258
#define CSR_UP  259
#define CSR_DN  260
#define CSR_RHT 261
#define CSR_LFT 262
#define END     263
#define NO_BEEP   0
#define BEEP_OK   1

/*
 * Number of directories to store info on.
 * 20 should be good enough...any higher and memory use will get quite high
 * In fact, 20 might be too high.
 */
#define SAVEDIRS 20

/*
 * Which menu are we currently in.
 */
#define NUMMENU 4
#define MENUSTD 0
#define MENUEXT 1
#define MENUMUL 2
#define MENUDES 3

/*
 * keywin mode
 */
#define KEYWIN_NORMAL 0
#define KEYWIN_MARKED 1

/*
 * Version and patchlevel info.
 */
#define PROG_NAME "files"
#define VERSION "2.1"
#define PATCHLEVEL "10"

/*
 * Pause after executing a system() call or not?
 */
#define NOPAUSE 0
#define PAUSE   1

/*
 * What type of sorting should be done?  Default, name, size or time
 */
#define SBYDFLT 0
#define SBYNAME 1
#define SBYSIZE 2
#define SBYTIME 3
#define SBYEXTN 4

/*
 * Should the \i code be expanded or not?
 */
#define NOICODE 0
#define ICODE   1

#if 0
#define SW_CP 0
#define SW_CPR 0
#define SW_BAK 1
#define SW_MV 2
#endif

/*
 * The maximum nuber of macros
 */
#define MAX_MACROS 17

/*
 * Home debug directory.
 * files attempts to cd to this directory to dump core before dying.
 * It only does this if it is compiled with DEBUG defined.
 */
#define HOME_DDIR "/home/pfratar"

/*
 * Configuration defines
 */
#define CT_QUIT 0
#define CT_RDEL 1
#define CT_NOVU 2
#define CT_HIDN 3
#define CT_BEEP 4
#define CT_DEST 5
#define CT_DEL  6

/*
 * Ultrix...
 */
#ifdef DEC
#ifndef S_ISLNK
#define S_ISLNK(m)      (((m)&S_IFMT) == S_IFLNK)
#endif
#ifndef erasechar
#define erasechar() (_tty.sg_erase)
#endif
#ifndef killchar
#define killchar() (_tty.sg_kill)
#endif
#ifndef baudrate
#define baudrate() (_tty.sg_ospeed)
#endif
#endif /* DEC */

#ifdef MIPS
#ifndef S_ISREG
#define S_ISREG(m)      (((m)&S_IFMT) == S_IFREG)
#endif
#ifndef S_ISDIR
#define S_ISDIR(m)      (((m)&S_IFMT) == S_IFDIR)
#endif
#ifndef S_ISLNK
#define S_ISLNK(m)      (((m)&S_IFMT) == S_IFLNK)
#endif
#endif

/*
 * NeXT...
 */
#ifdef __NeXT__
typedef unsigned short mode_t;
typedef short nlink_t;
  #ifndef S_ISLNK
    #define S_ISLNK(m)      (((m)&S_IFMT) == S_IFLNK)
  #endif
  #ifndef S_ISDIR
    #define S_ISDIR(m)      (((m)&S_IFMT) == S_IFDIR)
  #endif
#endif

/*
 * A small kludge to set every pointer that is freed to (nil) once it is
 * freed.  RealFree() returns a (void *)0 if the free() works.
 */
#define Free(x) x = (void *)RealFree(x)

#define VOID_DECL void

#endif /* #ifndef __Defs_h__ */

