/*
 * $Source: /u/pfratar/src/files-2.1/src/RCS/dir.c,v $
 *
 * $Id: dir.c,v 6.15 1993/03/30 22:12:20 pfratar v2_1pl10 $
 *
 * files - full screen file manager for UNIX
 * Copyright (C) 1993 Paul Frattaroli
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
#define EXTERN
#include "global.h"
#undef EXTERN

#ifndef lint
static char *rcsid = "$Id: dir.c,v 6.15 1993/03/30 22:12:20 pfratar v2_1pl10 $";
#endif

#if defined(DEC) || defined(AIX) || defined(IRIX)
char *
realpath(char *op, char *rp)
{
    char sp[MAXPATHLEN];

    if(!getwd(sp))
        return(NULL);
    if(chdir(op))
        return(NULL);
    if(!getwd(rp))
        return(NULL);
    if(chdir(sp))
        return(NULL);
    return(rp);
}
#endif

/*
 * read_cur_dir() - Reads the current directory contained in cur_wd
 *                  and sets up all the appropriate data structures.
 */
void
read_cur_dir(char *filt)
{
    DIR *dirp;
    int i, blocks = 1;
    u_long bsz = BLOCKSIZE;
    struct entry *ee;
    char *dircont;
#ifdef AIX
    struct dirent *dp;
    dp = (struct dirent *)Alloc(sizeof(struct dirent));
#else
    struct direct *dp;
    dp = (struct direct *)Alloc(sizeof(struct direct));
#endif

    dircont = (char *)Alloc(BLOCKSIZE * FLSIZE * blocks * sizeof(char));

    /*
     * Open and read directory
     */
    count = 0;

    if(filt) {
        FILE *pr;
        char line[MAXPATHLEN];

        sprintf(line, "ls -1ad %s | uniq", filt);
        pr = popen(line, "r");
        while(fgets(line, MAXPATHLEN, pr)) {
            line[strlen(line) - 1] = '\0';
            if(!config_table[CT_HIDN].ct_val && *line == '.')
                continue;
            if(strcmp(line, ".") && strcmp(line, "..")) {
                strcpy((dircont + (count * FLSIZE)), line);
                count++;
                if(count >= bsz) {
                    blocks++;
                    bsz = blocks * BLOCKSIZE;
                    dircont = (char *)realloc(dircont,
                              (blocks * BLOCKSIZE * FLSIZE));
                    if(!dircont) {
                        warn_user("Could not allocate memory. Aborting");
                        exit_program();
                    }
                }
            }
        }
        pclose(pr);
    } else {
        dirp = opendir(cur_wd);
        while((dp = readdir(dirp)) != NULL) {
            if(!config_table[CT_HIDN].ct_val && *(dp->d_name) == '.')
                continue;
            if(strcmp(dp->d_name, ".") && strcmp(dp->d_name, "..")) {
                strcpy((dircont + (count * FLSIZE)), dp->d_name);
                count++;
                if(count >= bsz) {
                    blocks++;
                    bsz = blocks * BLOCKSIZE;
                    dircont = (char *)realloc(dircont,
                              (blocks * BLOCKSIZE * FLSIZE));
                    if(!dircont) {
                        warn_user("Could not allocate memory. Aborting");
                    }
                }
            }
        }
        closedir(dirp);
    }
    Free(dp);

    /*
     * Allocate memory for entries structs and marked_list
     */
    entries = (struct entry *)Alloc(count * sizeof(struct entry));
    marked_list = (int *)Alloc(count * sizeof(int));
    ee = entries;

    /*
     * Set up entries
     */
    if(count == 0) {
        entries = (struct entry *)Alloc(sizeof(struct entry));
        marked_list = (int *)Alloc(sizeof(int));
        ee = entries;

        /*
         * Since '/' is not a legal character in a file name, there
         * should be no way for the following to appear in the window
         * unless there really are no files in the directory.
         */
        ee->e_name = "\\\\ NO FILES //";
        ee->e_flstr = "  \\\\ NO FILES //";
        SET_FLG(ee->e_flags, EMP_DIR);
        count = 1;
    } else {
        for(i = 0; i < count; i++, ee++) {
            ee->e_name = (char *)Alloc(FLSIZE * sizeof(char));
            strcpy(ee->e_name, (dircont + (i * FLSIZE)));
            ee->e_flstr = (char *)Alloc(FLSIZE * sizeof(char));
            lstat((dircont + (i * FLSIZE)), &ee->e_stat);
            fl_str(ee);
            ee->e_flags = 0;
        }
    }
    Free(dircont);
}

/*
 * check_dirperms() - Make sure the directory is both readable and
 *                    searchable
 */
int
check_dirperms(char *path)
{
    DIR *dirp;

    if(!(dirp = opendir(path)))
        return 0;
    else
        closedir(dirp);
    return 1;
}

/*
 * new_dir() - Change into a new directory
 */
void
new_dir(char *path, char *filt)
{
    char rpath[MAXPATHLEN];

    if(!path) path = ".";
    realpath(path, rpath);
    if(config_table[CT_NOVU].ct_val &&
            strncmp(homedir, rpath, strlen(homedir))) {
        warn_user("Could not 'cd' into directory, permission denied");
        path = ".";
    }

    if(!check_dirperms(path)) {
        if(!strcmp(path, ".")) {
            print_error("Current directory is not readable. Aborting! ");
            exit_program();
        }
        print_error("Directory is not readable.");
        path = ".";
    }

    if(chdir(path) == -1) {
        print_error("Could not 'cd' into directory.");
        if(!strcmp(path, ".")) {
            print_error("Current directory does not exist.");
            warn_user("Attempting to 'cd' to your home.");
            if(chdir(homedir) == -1) {
                print_error("Could not 'cd' into directory.");
                warn_user("Attempting to 'cd' to \"/\".");
                if(chdir("/") == -1) {
                    print_error("Can't 'cd' to \"/\". Has the machine died?");
                    print_error("Maybe I'm dead...  or dreaming.");
                    print_error("Wait...  I see a light...");
                    exit_program();
                }
            }
        }
    }

    getwd(cur_wd);

    if(check_dirinfo(filt))
        set_fl_indices(fl_first, NOWARP);
    else {
        set_fl_indices(0, TOTOP);
        num_marked = 0;
        keywin_mode = KEYWIN_NORMAL;
    }

    switch(sort_type) {
        case SBYEXTN:
            qsort(entries, count, sizeof(struct entry), fl_extn_sort);
            break;
        case SBYTIME:
            qsort(entries, count, sizeof(struct entry), fl_time_sort);
            break;
        case SBYSIZE:
            qsort(entries, count, sizeof(struct entry), fl_size_sort);
            break;
        case SBYNAME:
            qsort(entries, count, sizeof(struct entry), fl_name_sort);
            break;
        default:
            qsort(entries, count, sizeof(struct entry), fl_def_sort);
    }

    set_menu();
    init_cwin();
    init_keywin();
    display_fwin();
}

/*
 * check_dirinfo() - Check to see if the new directory we are changing
 *                   into is already in the list of directories we have
 *                   info about.
 */
int
check_dirinfo(char *filt)
{
    int i, FOUND, ret, NEWFILT = FALSE;
    struct stat *tmpst;

    FOUND = -1;
    ret = 0;
    if(strcmp(filt, curfilter)) {
        free_all_info();
        NEWFILT = TRUE;
    }
    for(i = 0; i < SAVEDIRS; i++) {
        if(dirinfo[i].d_name && !strcmp(cur_wd, dirinfo[i].d_name)) {
            FOUND = i;
        }
    }
    if(FOUND != -1) {
        /*
         * Save the old directory.
         */
        save_old_dir(&dirinfo[dirindex]);
        dirindex = FOUND;

        /*
         * Restore from old dirinfo
         */
        sort_type = dirinfo[dirindex].d_sort_type;
        count = dirinfo[dirindex].d_count;
        fl_first = dirinfo[dirindex].d_fl_first;
        fl_last = dirinfo[dirindex].d_fl_last;
        cursor_pos = dirinfo[dirindex].d_cursor_pos;
        entries = dirinfo[dirindex].d_entries;
        num_marked = dirinfo[dirindex].d_num_marked;
        marked_list = dirinfo[dirindex].d_marked_list;
        keywin_mode = dirinfo[dirindex].d_kw_mode;
        marked_filesum = dirinfo[dirindex].d_mrk_sum;
        if(num_marked)
            NONE_MARKED = 0;

        /*
         * Stat the directory again
         */
        tmpst = (struct stat *)Alloc(sizeof(struct stat));
        stat(cur_wd, tmpst);

        /*
         * If the directory has been modified since the last time we
         * were here, read it again.
         */
        if(tmpst->st_mtime > dirinfo[dirindex].d_stat->st_mtime || NEWFILT) {
            free_old_dir(&dirinfo[dirindex]);
            if(*filt != '\0')
                read_cur_dir(filt);
            else
                read_cur_dir(0);
            dirinfo[dirindex].d_stat = tmpst;
            dirinfo[dirindex].d_name =
                (char *)Alloc(MAXPATHLEN * sizeof(char));
            strcpy(dirinfo[dirindex].d_name, cur_wd);
        } else {
            Free(tmpst);
        }
        ret = 1;
    } else {
        /*
         * The new directory wasn't found in the info we have so we
         * throw away the oldest.
         */
        if(dirindex != -1)
            save_old_dir(&dirinfo[dirindex]);
        do {
            next_dirindex++;
            next_dirindex %= SAVEDIRS;
        } while(TST_FLG(dirinfo[next_dirindex].d_flags, SAV_DIR) ||
                dirindex == next_dirindex);
        dirindex = next_dirindex;

        free_old_dir(&dirinfo[dirindex]);
        if(*filt != '\0')
            read_cur_dir(filt);
        else
            read_cur_dir(0);
        tmpst = (struct stat *)Alloc(sizeof(struct stat));
        stat(cur_wd, tmpst);
        dirinfo[dirindex].d_stat = tmpst;
        dirinfo[dirindex].d_name = (char *)Alloc(MAXPATHLEN * sizeof(char));
        strcpy(dirinfo[dirindex].d_name, cur_wd);
    }
    if(NEWFILT)
        strcpy(curfilter, filt);
    return(ret);
}

/*
 * save_old_dir() - save all the needed variables into the dirst struct
 *                  dip.
 */
void
save_old_dir(struct dirst *dip)
{
    /*
     * d_name and d_stat get set in check_dirinfo()
     */
    dip->d_sort_type = sort_type;
    dip->d_count = count;
    dip->d_fl_first = fl_first;
    dip->d_fl_last = fl_last;
    if(NONE_MARKED)
            num_marked = 0;
    dip->d_num_marked = num_marked;
    dip->d_marked_list = marked_list;
    dip->d_cursor_pos = cursor_pos;
    dip->d_entries = entries;
    if(num_marked <= 0)
        keywin_mode = KEYWIN_NORMAL;
    dip->d_kw_mode = keywin_mode;
    dip->d_mrk_sum = marked_filesum;
}

/*
 * free_old_dir() - Frees all the space allocated for the old directory
 *                  info in dip.
 */
void
free_old_dir(struct dirst *dip)
{
    int i;

    if(!dip)
        return;
    Free(dip->d_name);
    Free(dip->d_stat);
    Free(dip->d_marked_list);
    if(dip->d_entries) {
        for(i = 0; i < dip->d_count; i++)
            Free((dip->d_entries + i)->e_flstr);
        Free(dip->d_entries->e_name);
        Free(dip->d_entries);
    }
    dip->d_count = dip->d_fl_first = dip->d_fl_last = 0;
    dip->d_cursor_pos = dip->d_num_marked = 0;
    dip->d_kw_mode = KEYWIN_NORMAL; dip->d_mrk_sum = 0;
    dip->d_sort_type = SBYDFLT;
    dip->d_flags = (u_long)0;
}

/*
 * restat_file() - The current file may have changed status without
 *                 affecting the current directory.  Refresh our
 *                 info about the current file.
 */
void
restat_file(struct entry *fep)
{
    if(TST_FLG(fep->e_flags, EMP_DIR))
        return;
    lstat(fep->e_name, &fep->e_stat);
    fl_str(fep);
}

/*
 * free_all_info() - Free all the info we have on all directories stored.
 */
void
free_all_info(void)
{
    int i;

    for(i = 0; i < SAVEDIRS; i++)
            free_old_dir(&dirinfo[i]);
    Free(entries);
    Free(marked_list);
}

void
waitkey(void)
{
    char c;

    puts("Press any key to continue...");
    fflush(stdout);
    c = getchar();
}

void
get_home_directory(void)
{
    char *tmp, old_dir[MAXPATHLEN];

    getwd(old_dir);
    if((tmp = getenv("HOME")) != NULL) {
        if(chdir(tmp) == -1) {
            fputs("Could not 'chdir' into home directory.\n", stderr);
            fflush(stderr);
            waitkey();
        } else {
            getwd(homedir);
        }
    } else {
        fputs("'HOME' environment variable not set.\n", stderr);
        fflush(stderr);
        waitkey();
    }
    if(chdir(old_dir) == -1) {
        fputs("ACK! No working directory. Aborting.\n", stderr);
        fflush(stderr);
        exit(1);
    }
}

