/*
 * $Source: /u/pfratar/src/files-2.1/src/RCS/display.c,v $
 *
 * $Id: display.c,v 6.17 1993/03/30 22:12:20 pfratar v2_1pl10 $
 *
 * files - full screen file manager for UNIX
 * Copyright (C) 1993 Paul Frattaroli
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
#define EXTERN
#define _Display_C_
#include "global.h"
#undef _Display_C_
#undef EXTERN

#ifndef lint
static char *rcsid = "$Id: display.c,v 6.17 1993/03/30 22:12:20 pfratar v2_1pl10 $";
#endif

/*
 * init_screen() - Initialize the entire screen
 */
void
init_screen(void)
{

    initscr();

    fwinlines = LINES - CWINSIZE - KEYWINSIZE;
    maxfname = COLS - ATTRIBWIDTH - DATETIMEWIDTH -
               SIZEWIDTH - SPACEWIDTH;
    cwin = newwin(CWINSIZE, COLS, 0, 0);
    fwin = newwin(fwinlines, COLS, CWINSIZE, 0);
    keywin = newwin(KEYWINSIZE, COLS, (CWINSIZE + fwinlines), 0);
    scrollok(fwin, TRUE);
    scrollok(cwin, FALSE);
    scrollok(keywin, FALSE);
    clearok(fwin, TRUE);
    clearok(cwin, TRUE);
    clearok(keywin, TRUE);
    leaveok(fwin, FALSE);
    leaveok(cwin, FALSE);
    leaveok(keywin, FALSE);
    cbreak();
    noecho();
    nonl();
    refresh_screen();
}

/*
 * end_screen() - Run delwin() on all open windows and then endwin()
 */
void
end_screen(void)
{
    if(keywin) {
        werase(keywin);
        wrefresh(keywin);
        delwin(keywin);
    }
    if(cwin) {
        werase(cwin);
        wrefresh(cwin);
        delwin(cwin);
    }
    if(fwin) {
        werase(fwin);
        wrefresh(fwin);
        delwin(fwin);
    }
    cwin = (WINDOW *)0;
    fwin = (WINDOW *)0;
    keywin = (WINDOW *)0;
    erase();
    endwin();
}

/*
 * start_win() - Return to curses after a shell escape or system() call.
 */
void
start_win(void)
{
    init_screen();
    init_cwin();
    init_keywin();
    set_fl_indices(fl_first, NOWARP);
    display_fwin();
    refresh_screen();
}

/*
 * refresh_screen() - Refresh all the windows
 */
void
refresh_screen(void)
{
    wrefresh(cwin);
    wrefresh(fwin);
    wrefresh(keywin);
}

void
refresh_fwin(void)
{
    wrefresh(fwin);
}

void
refresh_cwin(void)
{
    wrefresh(cwin);
}

/*
 * init_cwin() - Initialize the command window
 */
void
init_cwin(void)
{
    char tmp[MAXPATHLEN];

    mymvwaddstr(cwin, 0, 0, cwin_ml1[menu_num]);
    wclrtoeol(cwin);
    mymvwaddstr(cwin, 1, 0, cwin_ml2[menu_num]);
    wclrtoeol(cwin);
    display_cwd();
    sprintf(tmp, hdfmtstr, "Attributes", "Size", "Date", "Time");
    mymvwaddstr(cwin, 3, 0, tmp);
    wrefresh(cwin);
}

/*
 * display_cwd() - Display cwd in command window
 */
void
display_cwd(void)
{
    char tmp[MAXPATHLEN];

    fix_cwd(tmp);
    mvwprintw(cwin, 2, 0, "%s", tmp);
    wclrtoeol(cwin);
}

/*
 * fix_cwd() - Fix the current working directory so that it will fit
 *             across the screen.
 */
void
fix_cwd(char *p)
{
    char *q;

    if(strlen(cur_wd) >= COLS) {
        strcpy(p, ".../");
        q = cur_wd;
        while(strlen(q) >= COLS - 5) {
            q = (char *)index(q, '/');
            q++;
        }
        strcat(p, q);
    } else {
        strcpy(p, cur_wd);
    }
}

#ifdef DEBUG
void
dump_win_st()
{
    FILE *df;

    df = fopen("/usr/u/pfratar/foo", "w");
    fprintf(df, "ROWS: %d  COLS: %d\n", LINES, COLS);
    fprintf(df, "CURSCR: 0x%08x  bx: %d  by: %d   ex: %d  ey: %d\n",
        curscr, curscr->_begx, curscr->_begy, curscr->_maxx, curscr->_maxy
    );
    fprintf(df, "FWIN: 0x%08x  bx: %d  by: %d   ex: %d  ey: %d\n",
        fwin, fwin->_begx, fwin->_begy, fwin->_maxx, fwin->_maxy
    );
    fprintf(df, "CWIN: 0x%08x  bx: %d  by: %d   ex: %d  ey: %d\n",
        cwin, cwin->_begx, cwin->_begy, cwin->_maxx, cwin->_maxy
    );
    fprintf(df, "KEYWIN: 0x%08x  bx: %d  by: %d   ex: %d  ey: %d\n",
        keywin, keywin->_begx, keywin->_begy, keywin->_maxx, keywin->_maxy
    );
    fclose(df);
}
#endif

/*
 * init_keywin() - Initialize the key help window
 */
void
init_keywin(void)
{
#if 1
    char kwh[MAXPATHLEN], *p;
    int i;

    sprintf(kwh, "---- Current filename filter: \"%s\" ", curfilter);
    i = COLS - 33 - strlen(curfilter);
    p = kwh;
    while(*p) p++;
    while(i-- > 0) *p++ = '-';
    *p = '\0';
    mymvwaddstr(keywin, 0, 0, kwh);
#else
    mymvwaddstr(keywin, 0, 0, kwl1);
#endif
    mymvwaddstr(keywin, 1, 0, kwl2);
    mymvwaddstr(keywin, 2, 0, kwl3);
    update_keywin();
    wrefresh(keywin);
}

void
update_keywin(void)
{
    if(keywin_mode == KEYWIN_MARKED) {
        mvwprintw(keywin, 1, 0, "Marked:  %lu bytes in %d file%s.\n",
            marked_filesum, num_marked,
            (num_marked == 1) ? "" : "s");
        wmove(keywin, 2, 0);
        wclrtoeol(keywin);
        wrefresh(keywin);
    }
}

/*
 * repaint_screen() - Cause the entire screen to be re-displayed.
 */
void
repaint_screen(void)
{
    werase(cwin);
    werase(fwin);
    werase(keywin);
    refresh_screen();
#if 0
    init_cwin();
    init_keywin();
    display_fwin();
#else
    free_old_dir(&dirinfo[dirindex]);
    new_dir(".", curfilter);
#endif
}

/*
 * page_up() - Scroll the screen up by a full page
 */
void
page_up(void)
{
    set_fl_indices(fl_first - fwinlines + 1, NOWARP);
    display_fwin();
}

/*
 * page_down() - Scroll the screen down by a full page
 */
void
page_down(void)
{
    set_fl_indices(fl_first + fwinlines - 1, NOWARP);
    display_fwin();
}

/*
 * scroll_up() - Scroll the screen up by a half-page
 */
void
scroll_up(void)
{
    set_fl_indices(fl_first - (fwinlines/2) + 1, NOWARP);
    display_fwin();
}

/*
 * scroll_down() - Scroll the screen down by a half-page
 */
void
scroll_down(void)
{
    set_fl_indices(fl_first + (fwinlines/2) - 1, NOWARP);
    display_fwin();
}

/*
 * cursor_up() - Move cursor up one line, scroll if necessary.
 */
int
cursor_up(int current, int beep)
{
    if(current == 0) {
        if(fl_first > 0) {
            wmove(fwin, 0, 0);
            winsertln(fwin);
            set_fl_indices(fl_first - 1, NOWARP);
            mvwaddstr(fwin, 0, 0, (entries + fl_first)->e_flstr);
        } else {
            if(beep)
                putchar('\007');
        }
        display_marks();
        return(0);
    }
    return(1);
}

/*
 * cursor_down() - Move cursor down one line, scroll if necessary.
 */
int
cursor_down(int current, int beep)
{
    if(current == fl_last - fl_first) {
        if(fl_last < count - 1) {
            scroll(fwin);
            set_fl_indices(fl_first + 1, NOWARP);
            mvwaddstr(fwin, fwinlines - 1, 0, (entries + fl_last)->e_flstr);
        } else {
            if(beep)
                putchar('\007');
        }
        display_marks();
        return(0);
    }
    return(1);
}

/*
 * go_top() - Go instantly (well as fast as curses can) to the top of
 *            the list.
 */
void
go_top(void)
{
    set_fl_indices(0, TOTOP);
    display_fwin();
}

/*
 * go_end() - Go instantly to the bottom of the list.
 */
void
go_end(void)
{
    set_fl_indices(count - (fwinlines/2) - 1, TOEND);
    display_fwin();
}

/*
 * display_fwin() - Re-write file window from fl_first to fl_last
 */
void
display_fwin(void)
{
    int i, x;

    for( i = fl_first; i <= fl_last; i++ ) {
        mvwaddstr(fwin, i - fl_first, 0, (entries + i)->e_flstr);
        wclrtoeol(fwin);
    }
    x = fl_last - fl_first;
    if( x < fwinlines ) {
        wmove(fwin, x + 1, 0);
        wclrtobot(fwin);
    }
    display_marks();
    wmove(fwin, cursor_pos, 0);
    wrefresh(fwin);
}

/*
 * display_marks() - Display the files which are currently marked in
 *                   this directory by turning on a '*' beside them.
 */
void
display_marks(void)
{
    int i;

    marked_filesum = 0;
    if(num_marked > 0)
        keywin_mode = KEYWIN_MARKED;
    else
        keywin_mode = KEYWIN_NORMAL;
    for( i = 0; i < num_marked; i++) {
        if(*(marked_list + i) >= fl_first && *(marked_list + i) <= fl_last)
            mvwaddch(fwin, (*(marked_list + i) - fl_first), 0, '*');
        marked_filesum += (entries + *(marked_list + i))->e_stat.st_size;
    }
    update_keywin();
}

/*
 * warn_user() - Print out a warning to the user and wait until they
 *               press a key before continuing.
 */
void
warn_user(char *warn)
{
    char chr;

    clear_cwin();
    mvwaddstr(cwin, 0, 0, warn);
    mvwaddstr(cwin, 2, 0, ENTRET_PROMPT);
    wrefresh(cwin);
    chr = wgetch(cwin);
    while(chr != '\n' && chr != '\r')
        chr = wgetch(cwin);
    wclear(cwin);
    init_cwin();
    wrefresh(cwin);
}

/*
 * clear_cwin() - Clear the command window except for the header line.
 */
void
clear_cwin(void)
{
    int i;

    for(i = 0; i < CWINSIZE - 1; i++) {
        wmove(cwin, i, 0);
        wclrtoeol(cwin);
    }
    wrefresh(cwin);
}

/*
 * clear_fwin() - Clear the file window.
 */
void
clear_fwin(void)
{
    wmove(fwin, 0, 0);
    wclrtobot(fwin);
    wrefresh(fwin);
}

/*
 * ask_user() - Ask the user the question 'ques' and get back a yes
 *              or no answer.
 */
int
ask_user(char *ques, char def)
{
    int rc, ty, tx;
    char chr;

    clear_cwin();
    mvwaddstr(cwin, 0, 0, ques);
    getyx(cwin, ty, tx);
    waddch(cwin, def);
    wclrtoeol(cwin);
    wmove(cwin, ty, tx);
    wrefresh(cwin);
    chr = wgetch(cwin);
    if(chr == '\n' || chr == '\r')
        chr = def;
    if(isupper(chr))
        chr = tolower(chr);
    rc = (chr == 'y');
    init_cwin();
    wrefresh(cwin);
    return(rc);
}

/*
 * get_char_ans() - Get a single character as an answer to 'ques' from
 *                  the user.
 */
int
get_char_ans(char *ques)
{
    int chr;

    clear_cwin();
    mvwaddstr(cwin, 0, 0, ques);
    wclrtoeol(cwin);
    wrefresh(cwin);
    chr = my_getch(cwin);
    if(isupper(chr) && chr < 256)
        chr = tolower(chr);
    init_cwin();
    wrefresh(cwin);
    return(chr);
}

/*
 * get_user_input() - Display the message 'ques' to the user and get
 *                    back a string of input in 'ans'.
 */
void
get_user_input(char *ques, char *ans)
{
    int i;

    mvwaddstr(cwin, 0, 0, ques);
    wclrtoeol(cwin);
    for(i = 1; i < CWINSIZE - 1; i++) {
        wmove(cwin, i, 0);
        wclrtoeol(cwin);
    }
    nocbreak();
    raw();
    wrefresh(cwin);
    wmove(cwin, 2, 0);
    mygetstr(cwin, ans);
    noraw();
    cbreak();
    init_cwin();
    wrefresh(cwin);
}

/*
 * mygetstr() - I couldn't get curses' mvwgetstr() to work properly
 *              so I had to write my own function to get a str of
 *              input from the user.
 */
void
mygetstr(WINDOW *ww, char *str)
{
    char k, e, *q;
    int tmpx, tmpy, chr;

    k = killchar();
    e = erasechar();
    q = str;
    chr = my_getch(ww);
    while(chr != '\n' && chr != '\r') {
        if( chr == k ) {
            getyx(ww, tmpy, tmpx);
            wmove(ww, tmpy, 0);
            wclrtoeol(ww);
            q = str;
        } else if( chr == e ) {
            getyx(ww, tmpy, tmpx);
            mvwdelch(ww, tmpy, tmpx - 1);
            q--;
            if(q < str)
                q = str;
        } else {
            switch(chr) {
                case '\022': /* ^R */
                    getyx(ww, tmpy, tmpx);
                    mvwaddstr(ww, tmpy, 0, str);
                    wrefresh(ww);
                    q = rindex(str, '\0');
                    break;
                case '\033':
                    *str = '\0';
                    return;
                    break;
                default:
                    if(isprint(chr)) {
                        waddch(ww, chr);
                        *q++ = chr;
                        *q = '\0';
                    }
            }
        }
        wrefresh(ww);
        chr = my_getch(ww);
    }
    *q = '\0';
}

/*
 * place_mark() - Place a marker beside the current file.
 */
void
place_mark(void)
{
    int ty, tx;

    waddch(fwin, '*');
    getyx(fwin, ty, tx);
    wmove(fwin, ty, tx - 1);
    wrefresh(fwin);
    if(num_marked == 0) {
        menu_num = MENUMUL;
        keywin_mode = KEYWIN_MARKED;
        init_cwin();
        init_keywin();
    }
    *(marked_list + num_marked) = fl_first + cursor_pos;
    marked_filesum += (u_long)(entries + fl_first + cursor_pos)->e_stat.st_size;
    num_marked++;
    update_keywin();
}

/*
 * remove_mark() - Remove the marker from beside a file.
 */
void
remove_mark(int which)
{
    int ty, tx, i;

    waddch(fwin, ' ');
    getyx(fwin, ty, tx);
    wmove(fwin, ty, tx - 1);
    wrefresh(fwin);
    for(i = which; i < num_marked - 1; i++)
        *(marked_list + i) = *(marked_list + i + 1);
    num_marked--;
    marked_filesum -= (u_long)(entries + fl_first + cursor_pos)->e_stat.st_size;
    if(num_marked < 0)
        num_marked = 0;
    if(num_marked == 0) {
        menu_num = MENUSTD;
        keywin_mode = KEYWIN_NORMAL;
        marked_filesum = 0;
        init_cwin();
        init_keywin();
    }
    update_keywin();
}

/*
 * set_menu() - Set the number of the menu to display.
 */
void
set_menu(void)
{
    if(menu_num != MENUDES) {
        if(num_marked)
            menu_num = MENUMUL;
        else
            menu_num = MENUSTD;
    }
}

/*
 * display_version() - Display the current version info
 */
void
display_version(void)
{
    char tmp[MAXPATHLEN];

    sprintf(tmp, "%s by Paul Frattaroli.  version: %s  patchlevel: %s\n%s",
            progname, VERSION, PATCHLEVEL, copyright);
    warn_user(tmp);
}

/*
 * cwin_yxaddstr() - Add a string to the command window at coords y, x.
 */
void
cwin_yxaddstr(int y, int x, char *str)
{
    mvwaddstr(cwin, y, x, str);
    wclrtoeol(cwin);
    wrefresh(cwin);
}

/*
 * cwin_move() - Move to a new location in the command window.
 */
void
cwin_move(int y, int x)
{
    wmove(cwin, y, x);
    wrefresh(cwin);
}

/*
 * fwin_yxaddstr() - Add a string to the file window at coords y, x.
 */
void
fwin_yxaddstr(int y, int x, char *str)
{
    mvwaddstr(fwin, y, x, str);
    wclrtoeol(fwin);
    wrefresh(fwin);
}

/*
 * fwin_wait() - Wait until a enter or return key is pressed in the
 *               file window.
 */
void
fwin_wait(void)
{
    char cc;

    cc = wgetch(fwin);
    while(cc != '\n' && cc != '\r')
        cc = wgetch(fwin);
}

/*
 * display_file_info() - Display info on the current file in the command
 *                       window.
 */
void
display_file_info(char *fn, char *o, char *g, ino_t inod, nlink_t numl,
                  char *atm, char *ctm)
{
    char cc;

    clear_cwin();
    mvwprintw(cwin, 0, 0, FINFOFMT, fn, o, g, inod, numl, atm, ctm);
    wrefresh(cwin);
    do {
        cc = wgetch(cwin);
    } while(cc != '\n' && cc != '\r');
    init_cwin();
}

/*
 * calc_fmt_strings() - Calculate the file list and header format
 *                      strings.
 */
void
calc_fmt_strings(void)
{
    int flw;
    char *p;

    if(!flfmtstr)
        flfmtstr = (char *)Alloc(MAXPATHLEN * sizeof(char));
    if(!hdfmtstr)
        hdfmtstr = (char *)Alloc(MAXPATHLEN * sizeof(char));
#if 0
    if(!kwl1)
        kwl1 = (char *)Alloc(MAXPATHLEN * sizeof(char));
#endif
    if(COLS < MIN_SCR_WDTH || LINES < MIN_SCR_HGHT) {
        end_screen();
        system("clear");
        fprintf(stderr,
            "%s: your screen is %d by %d.\n%s requires at least %d by %d.\n",
            progname, LINES, COLS, progname,
            MIN_SCR_HGHT, MIN_SCR_WDTH
        );
        exit(1);
    }
    flw = COLS - ATTRIBWIDTH - DATETIMEWIDTH - SIZEWIDTH - SPACEWIDTH;
    sprintf(flfmtstr, "  %%-%ds%s", flw + 1, FLFORMAT);

#ifdef BOLDING
    strcpy(hdfmtstr, "- \\HName\\h ");
#else
    strcpy(hdfmtstr, "- Name ");
#endif
    p = (char *)rindex(hdfmtstr, '\0');
    flw -= NAME_HD_SIZE;
    while(flw >= 0) {
        *p = '-';
        p++;
        flw--;
    }
    *p = '\0';
    strcat(hdfmtstr, HDFORMAT);

#if 0
    p = kwl1;
    for(flw = 0; flw < COLS; flw++)
        *p = '-', p++;
    *p = '\0';
#endif
}

/*
 * window_change_size() - Signal handler for SIGWINCH signal.
 */
void
window_change_size(void)
{
    free_all_info();
    end_screen();
    ioctl(0, TIOCGWINSZ, &win_ws);
    if(win_ws.ws_row) LINES = win_ws.ws_row;
    if(win_ws.ws_col) COLS = win_ws.ws_col;
    init_screen();
    calc_fmt_strings();
    set_fl_indices(fl_first, TOTOP);
    init_keywin();
    new_dir(".", curfilter);
}

/*
 * my_getch() - A wrapper for the getch() and wgetch() curses macros.
 */
int
my_getch(WINDOW *w)
{
    char c;
    int escstat, CONTINUE, ret;

    CONTINUE = 1;
    escstat = 0;

    while(CONTINUE) {
        if(w)
            c = wgetch(w);
        else
            c = getch();
        ret = (int)c;

#ifdef VTKEYS
        if(escstat)
            ret = escape_seq(&escstat, ret);
        if(escstat != 0)
            continue;
#endif

        switch(ret) {
#ifdef VTKEYS
            case '\033':
#if 0
                escstat = CONTINUE = pending_input();
#else
                escstat = CONTINUE = 1;
#endif
                break;
#endif /* VTKEYS */
            default:
                CONTINUE = 0;
                break;
        }
    }
    return(ret);
}

#if 0
/*
 * pending_input() - Check if there is pending input on stdin.
 */
int
pending_input(void)
{
    fd_set stuff;
    struct timeval  tv;

    /*
    * 1 second seems like a lot, but the BSD source for ex/vi
    * uses alarm instead of select with a timeout of 1 second.
    * So, if 1 second is good enough for vi...
    */
    tv.tv_sec  = 1;
    tv.tv_usec = 0;

    FD_ZERO(&stuff);
    FD_SET(0, &stuff);
    switch(select(FD_SETSIZE, &stuff, (fd_set *)0, (fd_set *)0, &tv)) {
        case -1:
            print_error("Error in select().");
            break;
        case 0:
            return 0;
            break;
    }
    return FD_ISSET(0, &stuff);
}
#endif

/*
 * escape_seq() - Handle escape sequences such as cursor keys.
 */
int
escape_seq(int *status, int chr)
{
    if(isupper(chr) && chr < 256)
        chr = tolower(chr);
    switch(*status) {
        case 1:
            switch(chr) {
                case ',': chr = PG_UP; break;   /* page up              */
                case '.': chr = PG_DN; break;   /* page down            */
                case 'h': chr = HOME; break;    /* top/home             */
                case 'a': chr = CSR_UP; break;  /* up arrow for vi50    */
                case 'b': chr = CSR_DN; break;  /* down arrow for vi50  */
                case 'c': chr = CSR_RHT; break; /* right arrow for vi50 */
                case 'd': chr = CSR_LFT; break; /* left arrow for vi50  */
            }
            *status = (chr == '[' || chr == 'o') ? 2 : 0;
            break;
        case 2:
            switch(chr) {
                case '2': *status = 3; break;    /* part end   */
                case 'a': chr = CSR_UP; break;   /* up         */
                case 'b': chr = CSR_DN; break;   /* down       */
                case 'c': chr = CSR_RHT; break;  /* select     */
                case 'd': chr = CSR_LFT; break;  /* up one dir */
                case 's': chr = PG_UP; break;    /* page up    */
                default: chr = (int)'.'; break;
            }
            *status = (*status == 3) ? 3 : 0;
            break;
        case 3:
            switch(chr) {
                case '~': chr = END; break;   /* end */
            }
            *status = 0;
            break;
    }
    return chr;
}

/*
 * set_isig() - set the isig flag on.  Calling cbreak() will set it
 *              off but in some cases we want both cbreak and isig.
 */
void
set_isig()
{
    struct termios tios;

    tcgetattr(0, &tios);
    tios.c_lflag |= ISIG;
    tcsetattr(0, TCSANOW, &tios);
}

/*
 * init_syswin() - Initialize the syswin
 */
void
init_syswin(void)
{
    syswin = newwin(CWINSIZE - 1, COLS, 0, 0);
    scrollok(syswin, TRUE);
    clearok(syswin, TRUE);
    leaveok(syswin, FALSE);
    cbreak();
    noecho();
    nonl();
    set_isig();
    wclear(syswin);
    touchwin(syswin);
    wrefresh(syswin);
}

/*
 * end_syswin() - Close down the syswin
 */
void
end_syswin(void)
{
    delwin(syswin);
    cbreak();
    noecho();
    nonl();
    set_isig();
}

/*
 * syswin_yxaddstr() - Add a string to the syswin window.
 */
void
syswin_addstr(char *str)
{
    waddstr(syswin, str);
    wclrtoeol(syswin);
    wrefresh(syswin);
}

/*
 * syswin_msg() - Print a message in the syswin window
 */
void
syswin_msg(char *warn)
{
    waddstr(syswin, warn);
    wrefresh(syswin);
}

/*
 * syswin_wait() - Wait for enter or return to be pressed in the syswin
 *                 window
 */
void
syswin_wait(void)
{
    char chr;

    waddstr(syswin, ENTRET_PROMPT);
    wrefresh(syswin);
    chr = wgetch(syswin);
    while(chr != '\n' && chr != '\r')
        chr = wgetch(syswin);
}

