/*
 * $Source: /u/pfratar/src/files-2.1/src/RCS/fileops.c,v $
 *
 * $Id: fileops.c,v 6.14 1993/03/30 22:12:20 pfratar v2_1pl10 $
 *
 * files - full screen file manager for UNIX
 * Copyright (C) 1993 Paul Frattaroli
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
#define EXTERN
#include "global.h"
#undef EXTERN

#ifndef lint
static char *rcsid = "$Id: fileops.c,v 6.14 1993/03/30 22:12:20 pfratar v2_1pl10 $";
#endif

/*
 * backup_file() - Back a file up to 'filename.bak'.
 */
int
backup_file()
{
    syswin_system(PATH_CP, (char *)0, "\\f.bak");
    num_marked = 0;
    new_dir(".", curfilter);
    return(RCNOERR);
}

/*
 * copy_file() - Copy the current file to another path/file.
 */
int
copy_file(int newnm, int indest)
{
    static char dest[MAXPATHLEN];
    static int nm, mnum;
    char nn[MAXPATHLEN];

    if(config_table[CT_DEST].ct_val) {
        *nn = 0;
        if(!indest) {
            nm = num_marked;
            mnum = -1;
            if(NONE_MARKED && num_marked == 1)
                mnum = *marked_list;
            menu_num = MENUDES;
            strcpy(copy_dir, cur_wd);
            SET_FLG(dirinfo[dirindex].d_flags, SAV_DIR);
            copy_cmd = TRUE;
        } else {
            if(newnm)
                get_user_input("Enter new name in current directory:", nn);
            sprintf(dest, "%s%s%s", cur_wd, (*nn) ? "/" : "", nn);
            new_dir(copy_dir, curfilter);
            num_marked = nm;
            if(mnum != -1)
                *marked_list = mnum;
            syswin_system(PATH_CP, "-r", expand_dir(dest));
            num_marked = 0;
            menu_num = MENUSTD;
            CLR_FLG(dirinfo[dirindex].d_flags, SAV_DIR);
        }
    } else {
        get_user_input("Enter destination (Press ^R to retrieve last):", dest);
        if(dest[0])
            syswin_system(PATH_CP, "-r", expand_dir(dest));
        num_marked = 0;
    }
    new_dir(".", curfilter);
    return(RCNOERR);
}

/*
 * create_file_dir() - Create (mkdir/touch) a new directory/file.
 */
int
create_file_dir()
{
    int cc;
    char new[MAXPATHLEN];
    FILE *fp;

    cc = ' ';
    new[0] = '\0';
    while( cc != 'f' && cc != 'd' ) {
        cc = get_char_ans("Create a new [f]ile or new [d]irectory? ");
        if( cc == '\n' || cc == '\r' || cc == '\033' )
            break;
    }
    switch(cc) {
        case 'f':
            get_user_input("Enter new file name:", new);
            if(*new) {
                fp = fopen(expand_dir(new), "a");
                if(fp)
                    fclose(fp);
                else
                    print_error("Could not create file:");
            }
            break;
        case 'd':
            get_user_input("Enter new directory name:", new);
            if(*new) {
                if(mkdir(expand_dir(new), (mode_t)0777))
                    print_error("Could not create directory:");
            }
            break;
    }
    num_marked = 0;
    new_dir(".", curfilter);
    return(RCNOERR);
}

/*
 * delete_file() - Delete the current file.
 */
int
delete_file()
{
    int i;
    mode_t x;
    char question[MAXPATHLEN];

    if(NONE_MARKED)
        sprintf(question, "Delete file: '%s'? (y/n) ",
                (entries + fl_first + cursor_pos)->e_name);
    else
        sprintf(question, "Delete marked files? (y/n) ");
#if 0
    if(!config_table[CT_DEL].ct_val || ask_user(question, 'n')) {
#else
    if(ask_user(question, 'n')) {
#endif
        for(i = 0; i < num_marked; i++) {
            x = (entries + *(marked_list + i))->e_stat.st_mode;
            x &= 0170000;
            x >>= 12;
            if(x == 004) {
                if(config_table[CT_RDEL].ct_val) {
                    syswin_system(PATH_RM, "-rf", (char *)0);
                } else {
                    if(rmdir((entries + *(marked_list + i))->e_name) == -1)
                        print_error("Could not remove directory.");
                }
            } else {
                if(unlink((entries + *(marked_list + i))->e_name) == -1)
                    print_error("Could not delete file.");
            }
        }
    }
    num_marked = 0;
    new_dir(".", curfilter);
    return(RCNOERR);
}

/*
 * edit_file() - Edit the current file using the users editor.
 */
int
edit_file()
{
    char cmd[MAXPATHLEN], editor[MAXPATHLEN], *tmp;

    if(S_ISREG((entries + fl_first + cursor_pos)->e_stat.st_mode) ||
            S_ISLNK((entries + fl_first + cursor_pos)->e_stat.st_mode)) {
        tmp = getenv("EDITOR");
        if(tmp) {
            strcpy(editor, tmp);
            sprintf(cmd, "%s \\a", editor);
        } else
            sprintf(cmd, "%s \\a", DEF_EDITOR);
        end_screen();
        preproc_cmd(cmd);
        expand_cmd(cmd, (struct entry *)0);
        my_system(cmd, NOPAUSE);
        restat_file((entries + fl_first + cursor_pos));
        num_marked = 0;
        start_win();
    } else {
        warn_user("That is not a regular file.  It cannot be edited.");
    }
    num_marked = 0;
    new_dir(".", curfilter);
    return(RCNOERR);
}

/*
 * filter_files() - Apply a filter to the current directory.
 */
int
filter_files()
{
    static char filter[MAXPATHLEN];

    get_user_input("Enter filter (Press ^R to retrieve last filter):", filter);
    new_dir(".", filter);
    return(RCNOERR);
}

/*
 * grep_file() - Search for a string in a file using egrep.
 */
int
grep_file()
{
    char cmd[MAXPATHLEN], search[MAXPATHLEN];

    search[0] = '\0';
    get_user_input("Enter a string to search for:", search);
    if(*search) {
        sprintf(cmd, "%s -e \"%s\" '%s'", PATH_GREP, search,
                (entries + fl_first + cursor_pos)->e_name);
        end_screen();
        expand_cmd(cmd, (entries + fl_first + cursor_pos));
        my_system(cmd, PAUSE);
        num_marked = 0;
        start_win();
    }
    num_marked = 0;
    new_dir(".", curfilter);
    return(RCNOERR);
}

/*
 * file_info() - Print detailed information about current file.  ie.
 *               more than what the standard display gives.
 */
int
file_info()
{
    struct tm *tt;
    struct passwd *pp;
    struct group *gg;
    char own[1024], grp[1024], atm[1024], ctm[1024], fnm[1024], tmp[1024];
    int nch;

    tt = (struct tm *)Alloc(sizeof(struct tm));
    /*
     * Get filename.
     */
    strcpy(fnm, (entries + fl_first + cursor_pos)->e_name);
    if(S_ISLNK((entries + fl_first + cursor_pos)->e_stat.st_mode)) {
        strcat(fnm, " -> ");
        nch = readlink((entries + fl_first + cursor_pos)->e_name,
                tmp, MAXPATHLEN);
        tmp[nch] = '\0';
        strcat(fnm, tmp);
    }

    /*
     * Get owner and group names.
     */
    pp = getpwuid((entries + fl_first + cursor_pos)->e_stat.st_uid);
    gg = getgrgid((entries + fl_first + cursor_pos)->e_stat.st_gid);
    strcpy(own, pp->pw_name);
    strcpy(grp, gg->gr_name);

    /*
     * Get access and creation times.
     */
#ifndef MALLOCDEBUG
    tt = localtime(&((entries + fl_first + cursor_pos)->e_stat.st_atime));
    tm2str(tt, atm);
    tt = localtime(&((entries + fl_first + cursor_pos)->e_stat.st_ctime));
    tm2str(tt, ctm);
#else
    strcpy(atm, "dd/mm/yy hh:mm");
    strcpy(ctm, "dd/mm/yy hh:mm");
#endif

    display_file_info(fnm, own, grp,
            (ino_t)(entries + fl_first + cursor_pos)->e_stat.st_ino,
            (nlink_t)(entries + fl_first + cursor_pos)->e_stat.st_nlink,
            atm, ctm);

    Free(tt);
    return(RCNOERR);
}

/*
 * rename_file() - Rename a file to different path/file
 */
int
rename_file(int newnm, int indest)
{
    static char dest[MAXPATHLEN];
    static int nm, mnum;
    char nn[MAXPATHLEN];
    char *cmd[5];

    if(config_table[CT_DEST].ct_val) {
        *nn = 0;
        if(!indest) {
            nm = num_marked;
            mnum = -1;
            if(NONE_MARKED && num_marked == 1)
                mnum = *marked_list;
            menu_num = MENUDES;
            strcpy(copy_dir, cur_wd);
            SET_FLG(dirinfo[dirindex].d_flags, SAV_DIR);
            copy_cmd = FALSE;
        } else {
            if(newnm)
                get_user_input("Enter new name in current directory:", nn);
            sprintf(dest, "%s%s%s", cur_wd, (*nn) ? "/" : "", nn);
            new_dir(copy_dir, curfilter);
            num_marked = nm;
            if(mnum != -1)
                *marked_list = mnum;
            syswin_system(PATH_MV, (char *)0, expand_dir(dest));
            num_marked = 0;
            menu_num = MENUSTD;
            CLR_FLG(dirinfo[dirindex].d_flags, SAV_DIR);
        }
    } else {
        get_user_input("Enter new name (Press ^R to retrieve last):", dest);
        if(*dest)
            syswin_system(PATH_MV, (char *)0, expand_dir(dest));
        num_marked = 0;
    }
    new_dir(".", curfilter);
    return(RCNOERR);
}

