/*
 * $Source: /u/pfratar/src/files-2.1/src/RCS/global.h,v $
 *
 * $Id: global.h,v 6.13 1993/03/30 22:12:20 pfratar v2_1pl10 $
 *
 * files - full screen file manager for UNIX
 * Copyright (C) 1993 Paul Frattaroli
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
#include <stdio.h>
#ifndef __NeXT__
#include <stdlib.h>
#include <string.h>
#else
#include <libc.h>
#endif
#include <strings.h>
#include <time.h>
#include <sys/types.h>

#ifdef MIPS
extern int errno;
typedef unsigned short  mode_t;
typedef short           nlink_t;
typedef int		pid_t;
#endif

#include <sys/dir.h>
#include <sys/param.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <ctype.h>
#include <fcntl.h>
#include <signal.h>
#include <errno.h>
#include <pwd.h>
#include <grp.h>
#include <curses.h>
#include "struct.h"
#include "defs.h"
#include "proto.h"
#include "rc.h"

#ifndef _Global_H_
#define _Global_H_

#ifndef EXTERN

    /*
     * all the windows used in files by curses
     */
    WINDOW		*cwin, *fwin, *syswin, *keywin;

    /* 
     * count of files in current dir
     * first and last files displayed in window
     * current cursor position in fwin
     * number of lines in fwin
     */  
    int			count, fl_first, fl_last, cursor_pos, fwinlines;

    /*
     * the entries in the current directory
     */
    struct entry 	*entries;

    /*
     * current directory name
     */
    char 		*cur_wd;
    char		*copy_dir;
    int			copy_cmd;

    /*
     * macros and print commands, strings and number of
     */
    char 		**macro_list, **print_list;
    int 		nmacros, nprint;

    /*
     * what type of sorting
     */
    int 		sort_type = SBYDFLT;

    /*
     * index letters for the macros and print commands
     */
    char		mletters[] = "abcdefghijklmnopqrst";

    /*
     * list of marked files and how many are marked
     */
    int			*marked_list, num_marked = 0;

    /*
     * were we in multiple mode or not?
     */
    int			NONE_MARKED = 0;

    /*
     * which menu to display in cwin
     */
    int			menu_num;

    /*
     * format strings for the file list and header
     */
    char		*flfmtstr, *hdfmtstr;

    /*
     * list of saved directories and indexes into dirinfo
     */
    struct dirst	dirinfo[SAVEDIRS];
    int			dirindex, next_dirindex;

    /*
     * maximum size of a file name displayable in fwin
     */
    int			maxfname;

    /*
     * possible read and write perm combinations
     */
    char		*rwperms[4] = { "--", "-w", "r-", "rw" };

    /*
     * what is our name and get our stat info
     */
    char		*progname;
    struct stat		progstat;

    /*
     * what is the user's home directory
     */
    char		homedir[MAXPATHLEN];

    /*
     * file descriptor to redirect stderr to temporarily
     */
    int			tmpstderrfd = -1;

    /*
     * file pointer to write error messages to from stderr
     */
    FILE		*errfile;

    /*
     * what mode is the keywin in?
     */
    int			keywin_mode = KEYWIN_NORMAL;

    /*
     * how many bytes are marked
     */
    u_long		marked_filesum = 0;

    /*
     * hold the current window size
     */
    struct winsize	win_ws;

    /*
     * bury a copyright string in the binary
     */
    char 		copyright[] = "Copyright (c) 1992 by Paul Frattaroli";

    /*
     * The current file filter
     */
    char		curfilter[MAXPATHLEN];

    /*
     * Strings that are used quite a bit. Why allocate space for them
     * more than once?
     */
    char		*FMT_START_PATH = "%s/%s/%s";
    char		*FILESDIR = ".files";
    char		*FILESSTARTUP = ".filesrc";
    char		*MACROFILE = "macros";
    char		*PRINTFILE = "print";
    char		*CONFIGFILE = "config";
    char		*ENTRET_PROMPT = "Press enter/return to continue...";
#else

    extern WINDOW *cwin, *fwin, *syswin, *keywin;
    extern count, fl_first, fl_last, cursor_pos, fwinlines;
    extern struct entry *entries;
    extern char *cur_wd;
    extern char *copy_dir;
    extern int copy_cmd;
    extern char **macro_list, **print_list;
    extern int nmacros, nprint;
    extern int sort_type;
    extern char mletters[];
    extern int *marked_list, num_marked;
    extern int NONE_MARKED;
    extern int menu_num;
    extern char *flfmtstr, *hdfmtstr;
    extern struct dirst dirinfo[];
    extern int dirindex, next_dirindex;
    extern int maxfname;
    extern char *rwperms[];
    extern char *progname;
    extern struct stat progstat;
    extern char homedir[];
    extern int tmpstderrfd;
    extern FILE *errfile;
    extern int keywin_mode;
    extern u_long marked_filesum;
    extern struct winsize win_ws;
    extern char copyright[];
    extern char curfilter[];
    extern conf_t config_table[];
    extern char *FMT_START_PATH;
    extern char *FILESDIR;
    extern char *FILESSTARTUP;
    extern char *MACROFILE;
    extern char *PRINTFILE;
    extern char *CONFIGFILE;
    extern char	*ENTRET_PROMPT;

#endif

/*
 * set, reset and test entries struct flags
 */
#define SET_FLG(x,y) ((x) |= (y))
#define CLR_FLG(x,y) ((x) &= ~(y))
#define TST_FLG(x,y) ((x) & (y))

/*
 * values for the dirst and entry struct flags
 */
#define EMP_DIR (1L << 0)	/* Directory is empty              */
#define SAV_DIR (1L << 1)	/* Do not free this directory info */

#endif /* ifndef _Global_H_ */

