/*
 * $Source: /u/pfratar/src/files-2.1/src/RCS/macro.c,v $
 *
 * $Id: macro.c,v 6.17 1993/03/30 22:12:20 pfratar v2_1pl10 $
 *
 * files - full screen file manager for UNIX
 * Copyright (C) 1993 Paul Frattaroli
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
#define EXTERN
#include "global.h"
#undef EXTERN

#ifndef lint
static char *rcsid = "$Id: macro.c,v 6.17 1993/03/30 22:12:20 pfratar v2_1pl10 $";
#endif

void
read_a_file(char *fn, char **l, int *nm)
{
    FILE *mfp;
    char line[MAXPATHLEN];

    if((mfp = fopen(fn, "r")) != NULL) {
        while(!feof(mfp) && *nm < MAX_MACROS) {
            *(l + *nm) = (char *)Alloc(MAXPATHLEN * sizeof(char));
            line[0] = '\0';
            fgets(line, MAXPATHLEN, mfp);
            line[strlen(line) - 1] = '\0';
            if(valid_cmd_string(line)) {
                strcpy(*(l + *nm), line);
                (*nm)++;
                /* Only read in MAX_MACROS */
                if(*nm == MAX_MACROS) break;
            } else {
                Free(*(l + *nm));
            }
        }
        fclose(mfp);
    }
}

void
read_startup_files(void)
{
    FILE *mfp;
    char c, line[MAXPATHLEN], fname[MAXPATHLEN], *tmp;
    DIR *dp;
    int newway = 0, freetmp = 0;

    nmacros = 0;
    macro_list = (char **)Alloc(MAX_MACROS * sizeof(char *));
    print_list = (char **)Alloc(MAX_MACROS * sizeof(char *));
    if(*homedir) {
        tmp = (char *)Alloc(MAXPATHLEN * sizeof(char));
        strcpy(tmp, homedir);
        freetmp = TRUE;
    } else {
        tmp = getenv("HOME");
    }
    if(tmp)
        strcpy(fname, tmp);
    if(fname[0]) {
        strcat(fname, "/");
        strcat(fname, FILESDIR);
        dp = opendir(fname);
        if(!dp) {
            if(errno == ENOENT) {
                puts("You do not have a '.files' directory in your home.");
                printf("This directory is required to store %s\n%s",
                       progname, "macro definitions,\n",
                       "print commands and configuration info");
                do {
                    printf("Shall I create it for you? (y/n) ");
                    c = getchar();
                } while(c != 'y' && c != 'n');
                getchar();
                if(c == 'y') {
                    if(mkdir(fname, 0700)) {
                        perror("Could not create '.files' directory");
                    } else {
                        sprintf(line, "%s/%s", fname, MACROFILE);
                        sprintf(fname, "%s/%s", tmp, FILESSTARTUP);
                        if(!rename(fname, line))
                            newway = TRUE;
                        line[0] = '\0';
                        printf("\n%s", ENTRET_PROMPT);
                        do {
                            c = getchar();
                        } while(c != '\r' && c != '\n');
                    }
                }
            }
        } else {
            closedir(dp);
            newway = TRUE;
        }
        if(newway)
            sprintf(fname, FMT_START_PATH, tmp, FILESDIR, MACROFILE);
        else
            sprintf(fname, "%s/%s", tmp, FILESSTARTUP);
        read_a_file(fname, macro_list, &nmacros);
        if(newway) {
            sprintf(fname, FMT_START_PATH, tmp, FILESDIR, PRINTFILE);
            read_a_file(fname, print_list, &nprint);
        }
    }
    read_configuration();
    if(freetmp)
        Free(tmp);
}

void
write_a_file(char *fn, char **l, int nlm)
{
    FILE *mfp;
    char cmd[MAXPATHLEN];
    int i;

    if((mfp = fopen(fn, "w")) != NULL) {
        for( i = 0; i < nlm && i < MAX_MACROS; i++ )
            fprintf(mfp, "%s\n", *(l + i));
        fflush(mfp);
        fclose(mfp);
    } else {
        sprintf(cmd, "Could not open '%s' for write", fn);
        warn_user(cmd);
    }
}

void
write_startup_file(void)
{
    char fname[MAXPATHLEN], cmd[MAXPATHLEN], *tmp;
    struct stat tmpst;
    int i, freetmp = FALSE;

    if(*homedir) {
        tmp = (char *)Alloc(MAXPATHLEN * sizeof(char));
        strcpy(tmp, homedir);
        freetmp = TRUE;
    } else {
        tmp = getenv("HOME");
    }
    if(tmp) {
        strcpy(fname, tmp);
        sprintf(fname, FMT_START_PATH, tmp, FILESDIR, MACROFILE);
        write_a_file(fname, macro_list, nmacros);
        sprintf(fname, FMT_START_PATH, tmp, FILESDIR, PRINTFILE);
        write_a_file(fname, print_list, nprint);
    } else {
        warn_user("Can't find home directory.  Please set HOME variable.");
    }
    if(freetmp) Free(tmp);
}

/*
 * valid_cmd_string() - Checks to see if there where any non-space
 *                      characters in the command string which was
 *                      entered.
 */
int
valid_cmd_string(char *test)
{
    char *p;
    int rc = 0;

    if(p = index(test, '#')) *p = '\0';
    while(*test && rc == 0) {
        if(!isspace(*test)) rc = 1;
        test++;
    }
    return(rc);
}

/*
 * preproc_cmd() - Pre-process a command line.  Evaluate \i codes and place
 *                 quotes '' around file names to literalize spaces.
 */
void
preproc_cmd(char *cmd)
{
    char *p1, *p2, *q, tmp[MAXPATHLEN], tt[MAXPATHLEN];
    int in_quotes = 0;

    strcpy(tmp, cmd);
    p1 = tmp;
    p2 = cmd;
    tt[0] = '\0';

    while(*p2) {
        switch(*p2) {
            case '\\':
                if(!in_quotes) {
                    *p1 = '\'';
                    p1++;
                    in_quotes = 1;
                }
                p2++;
                switch(*p2) {
                    case 'i':
                        get_user_input("Enter user input:", tt);
                        q = tt;
                        while(*q) *p1++ = *q++;
                        break;
                    default:
                        *p1++ = '\\';
                        *p1++ = *p2;
                }
                p2++;
                if(in_quotes /* && *p2 != '\\' */) {
                    *p1++ = '\'';
                    in_quotes = 0;
                }
                break;
#if 0
            case '\'':
                *p1++ = '\\';
                *p1++ = '\'';
                p2++;
                break;
#endif
            default:
                *p1++ = *p2++;
        }
    }
    *p1 = '\0';
    strcpy(cmd, tmp);
}

/*
 * expand_cmd() - expands a command line and substitutes for the various
 *                codes which may be present.
 */
int
expand_cmd(char *cmd, struct entry *eptr)
{
    char *p1, *p2, tmp[MAXPATHLEN], tt[MAXPATHLEN];
    int i, rc = 0;

    p1 = tmp;
    p2 = cmd;

    /*
     * Scan cmd string and expand codes into cmd.
     */
    while(*p2) {
        switch(*p2) {
            case '\\':
                p2++;
                switch(*p2) {
                    case 'a':
                        for(i = 0; i < num_marked; i++) {
                            char *cp;

                            cp = (entries + *(marked_list + i))->e_name;
                            while(*cp)
                                *p1++ = *cp++;
                            *p1 = '\0';
                            if(strlen(tmp) > MAXPATHLEN - 50) {
                                i = num_marked;
                                tmp[0] = '\0';
                            }
                            if(i < num_marked - 1) {
                                *p1++ = '\'';
                                *p1++ = ' ';
                                *p1++ = '\'';
                            }
                        }
                        num_marked = 0;
                        break;
                    case 'e':
                        /*
                         * This is quite rude if the file doesn't really
                         * have an extension.  The problem is how to save
                         * the user from their own worst enemy...themself
                         */
                        *p1 = '\0';
                        strcpy(tt, fix_file_name(eptr->e_name));
                        p1 = index(tt, '.');
                        if(p1)
                            strcat(tmp, p1);
                        p1 = rindex(tmp, '\0');
                        rc = 1;
                        break;
                    case 'f':
                        *p1 = '\0';
                        strcat(tmp, fix_file_name(eptr->e_name));
                        p1 = rindex(tmp, '\0');
                        rc = 1;
                        break;
                    case 'F':
                        *p1 = '\0';
                        strcpy(tt, fix_file_name(eptr->e_name));
                        p1 = index(tt, '.');
                        if(p1)
                            *p1 = '\0';
                        strcat(tmp, tt);
                        p1 = rindex(tmp, '\0');
                        rc = 1;
                        break;
                    case 'p':
                        *p1 = '\0';
                        getwd(tt);
                        strcat(tt, "/");
                        strcat(tt, fix_file_name(eptr->e_name));
                        strcat(tmp, tt);
                        p1 = rindex(tmp, '\0');
                        break;
                    case 'P':
                        *p1 = '\0';
                        getwd(tt);
                        strcat(tt, "/");
                        strcat(tmp, tt);
                        p1 = rindex(tmp, '\0');
                        break;
                    case '\\':
                        *p1 = *p2;
                        p1++;
                        break;
                    default:
                        *p1 = '\\';
                        p1++;
                        *p1 = *p2;
                        p1++;
                }
                p2++;
                break;
            default:
                *p1 = *p2;
                p1++;
                p2++;
        }
    }
    *p1 = '\0';
    strcpy(cmd, tmp);
    return(rc);
}

/*
 * fix_file_name() - Fix a file name for use on the command line from FILES.
 *                   This involves escaping any single quotes (') which
 *                   may already exist in the file name.
 */
char *
fix_file_name(char *fn)
{
    static char nfn[MAXPATHLEN];
    char *p, *q;

    p = fn;
    q = nfn;
    bzero(q, MAXPATHLEN);
    while(*p) {
        if(*p == '\'') {
            *q++ = '\'';
            *q++ = '\\';
            *q++ = '\'';
            *q = '\'';
#if 0
        } else if (*p == '"') {
            *q++ = '\\';
            *q = '"';
#endif
        } else {
            *q = *p;
        }
        q++;
        p++;
    }
    *q = '\0';
    return(nfn);
}

/*
 * define_macro() - Define and edit your macros.
 */
int
define_macro(void)
{
    int i, mc, *nml;
    char mac_line[MAXPATHLEN], tmp[MAXPATHLEN], **ml, cc;

    cc = ' ';
    while( cc != 'm' && cc != 'p' ) {
        cc = get_char_ans("Define [m]acro or [p]rint command? ");
        if( cc == '\n' || cc == '\r' || cc == '\033' )
            return(RCNOERR);
    }

    if(cc == 'm') {
        ml = macro_list;
        nml = &nmacros;
    } else {
        ml = print_list;
        nml = &nprint;
    }
    
    clear_fwin();
    for(i = 0; i < *nml; i++) {
        sprintf(mac_line, "%c: %s", mletters[i], *(ml + i));
        fwin_yxaddstr(i, 0, mac_line);
    }
    mc = get_char_ans(
         "Press letter to edit OR enter/return to define a new macro: ");
    if( mc == '\n' || mc == '\r' ) {
        if(*nml < MAX_MACROS) {
            tmp[0] = '\0';
            get_user_input("Type in the new macro:", tmp);
            if(*tmp != '\0' && valid_cmd_string(tmp)) {
                *(ml + *nml) =
                        (char *)Alloc(MAXPATHLEN * sizeof(char));
                strcpy(*(ml + *nml), tmp);
                (*nml)++;
            }
        } else {
            warn_user("You have the maximum number of macros defined already.");
        }
    } else if( mc >= 'a' && mc < (*nml + 'a') ) {
        mc -= 'a';
        tmp[0] = '\0';
        get_user_input("Type in the new definition, or leave blank to delete:",
                tmp);
        if(*tmp == '\0' || !valid_cmd_string(tmp)) {
            Free(*(ml + mc));
            for( i = mc; i < *nml - 1; i++ ) {
                *(ml + i) = *(ml + i + 1);
            }
            *nml -= 1;
            if(*nml < 0)
                *nml = 0;
        } else {
            strcpy(*(ml + mc), tmp);
        }
    } else if( mc != '\033') {
        sprintf(mac_line, "Error: Macro '%c' not defined.", mc);
        warn_user(mac_line);
    }
    write_startup_file();
    num_marked = 0;
    new_dir(".", curfilter);
    return(RCNOERR);
}

/*
 * execute_macro() - execute a macro or print command.
 */
void
execute_macro(char **mpl, int nmp)
{
    char mac_line[MAXPATHLEN], cmd[MAXPATHLEN], ocmd[MAXPATHLEN];
    int mc, i;

    if(!nmp) {
        warn_user("No macros/commands defined.");
        return;
    }
    clear_fwin();
    for(i = 0; i < nmp; i++) {
        sprintf(mac_line, "%c: %s", mletters[i], *(mpl + i));
        fwin_yxaddstr(i, 0, mac_line);
    }
    if(mpl == print_list && nmp == 1)
        mc = 'a';
    else
        mc = get_char_ans("Type the letter of the macro to run: ");
    if(mc >= 'a' && mc < 'a' + nmp) {
        mc -= 'a';
        strcpy(cmd, *(mpl + mc));
        preproc_cmd(cmd);
        strcpy(ocmd, cmd);
        end_screen();
        for(i = 0; i < num_marked; i++) {
            expand_cmd(cmd, (entries + *(marked_list + i)));
            if(cmd[0]) {
                if( i == num_marked - 1 || num_marked == 0 )
                    my_system(cmd, PAUSE);
                else
                    my_system(cmd, NOPAUSE);
                restat_file((entries + *(marked_list + i)));
            } else {
                fprintf(stderr, "Command line too long\n");
                waitkey();
            }
            strcpy(cmd, ocmd);
        }
        num_marked = 0;
        start_win();
    } else if(mc != '\033' && mc != '\r' && mc != '\n') {
        sprintf(mac_line, "Error: Command '%c' not defined.", mc);
        warn_user(mac_line);
    }
}

/*
 * print_file() - Send a file to the printer using lpr.
 */
int
print_file(void)
{
    execute_macro(print_list, nprint);
    num_marked = 0;
    new_dir(".", curfilter);
    return(RCNOERR);
}

/*
 * your_cmd() - Run one of your macros.
 */
int
your_cmd(void)
{
    execute_macro(macro_list, nmacros);
    num_marked = 0;
    new_dir(".", curfilter);
    return(RCNOERR);
}

