/*
 * $Source: /u/pfratar/src/files-2.1/src/RCS/main.c,v $
 *
 * $Id: main.c,v 6.24 1993/04/14 19:23:53 pfratar v2_1pl10 $
 *
 * files - full screen file manager for UNIX
 * Copyright (C) 1993 Paul Frattaroli
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
#include "global.h"

#ifndef lint
static char *rcsid = "$Id: main.c,v 6.24 1993/04/14 19:23:53 pfratar v2_1pl10 $";
#endif

extern conf_t config_table[];
extern char *sys_errlist[];
pid_t cpid;
#define setstderr() dup2(tmpstderrfd, 2)
#define seterrfile() rewind(errfile); dup2(fileno(errfile), 2)

/*
 * main() - The main function.
 */
int
main(int argc, char *argv[])
{
    int chr, xx, rc = 0;
    int CONTINUE = 1;
    char *srules;

    cur_wd = (char *)Alloc(MAXPATHLEN * sizeof(char));
    getwd(cur_wd);
    copy_dir = (char *)Alloc(MAXPATHLEN * sizeof(char));

    if(argc > 1) {
        if(chdir(argv[1]) == -1) {
            fprintf(stderr, "%s: Could not chdir into %s.", argv[0], argv[1]);
            sleep(1);
            chdir(cur_wd);
        } else {
            getwd(cur_wd);
        }
    }

    /*
     * Try to stat ourself from argv[0], if not try each element in
     * the search rules.
     */
    if(stat(argv[0], &progstat)) {
        srules = getenv("PATH");
        if(srules) {
            char *pp, *qq;
            char pathp[MAXPATHLEN], paths[MAXPATHLEN];
            int MORE_PATHS = TRUE;

            strcpy(paths, srules);
            pp = qq = paths;
            while(MORE_PATHS) {
                while(*qq && *qq != ':') qq++;
                if(!*qq)
                    MORE_PATHS = FALSE;
                *qq = '\0';
                strcpy(pathp, pp);
                strcat(pathp, "/");
                strcat(pathp, argv[0]);
                if(!stat(pathp, &progstat))
                    MORE_PATHS = FALSE;
                pp = ++qq;
            }
        }
    }

    progname = basename(argv[0]);

    printf("%s version %s patchlevel %s, %s\n", progname, VERSION,
           PATCHLEVEL, copyright);
    puts("This program comes with ABSOLUTELY NO WARRANTY");
    puts("This is free software, and you are welcome to redistribute it");
    puts("under certain conditions.  See the file COPYING for more details\n");
    sleep(2);
    system("clear");

    if(!getenv("TERM")) {
        if(!getenv("TERMCAP")) {
            fprintf(stderr, "%s\n%s%s%s\n%s\n%s\n%s\n",
                "Someone hasn't set their TERM and TERMCAP variables.",
                "If you do not set these, ",
                progname,
                " may not work properly.",
                "Please set these variables using 'tset' or 'setterm'.",
                "See 'man tset' or 'man setterm' for more information.",
                "If you are using 'csh', you can try the following:"
            );
            fprintf(stderr, "\n%s\n%s\n%s\n\n    %s\n",
                "% set noglob",
                "% eval `tset -s type`",
                "% unset noglob",
                "where 'type' is the type of terminal you are using. eg. vt100"
            );
            exit(1);
        }
    }
    if(!isatty(0)) {
        fprintf(stderr, "%s: stdin is not a tty! - Aborting\n", progname);
        exit(1);
    }

    /*
     * 'root' could inadvertently do grievous damage with this program.
     * Warn user if we are 'root'
     */
    if(!geteuid()) {
        char ans;

        printf("WARNING: You are running %s as 'root'.\n", progname);
        printf("Continue? (y/n) ");
        do {
            ans = getchar();
            if(isupper(ans))
                ans = tolower(ans);
        } while(ans != 'n' && ans != 'y');
        if(ans == 'n')
            exit(0);
        ans = getchar(); /* get rid of extra \n or \r */
    }

    get_home_directory();
    read_startup_files();

    /*
     * Set up signal handlers.
     */
    signal(SIGWINCH, window_change_size);
    signal(SIGINT, exit_program);
    signal(SIGQUIT, exit_program);
    signal(SIGSEGV, handle_sigsegv);

    init_screen();
    calc_fmt_strings();

    dirindex = -1;
    next_dirindex = -1;
    cursor_pos = 0;

    setup_stderr();

    curfilter[0] = '\0';
    new_dir(".", curfilter);

    wmove(fwin, cursor_pos, 0);
    refresh_fwin();


    while(CONTINUE) {
        chr = my_getch(fwin);
        if(isupper(chr) && chr < 256)
            chr = tolower(chr);

        switch(chr) {
            case '<':
            case '\002': /* ^B */
            case PG_UP:
                page_up();
                break;
            case '\004': /* ^D */
                scroll_down();
                break;
            case '\005': /* ^E */
            case END:
                go_end();
                break;
            case '>':
            case '\006':
            case PG_DN: /* ^F */
                page_down();
                break;
            case '\014': /* ^L */
                repaint_screen();
                break;
            case '\024': /* ^T */
            case HOME:
                go_top();
                break;
            case '\025': /* ^U */
                scroll_up();
                break;
            case 'j':
            case CSR_DN:
                cursor_pos += cursor_down(cursor_pos,
                    config_table[CT_BEEP].ct_val);
                break;
            case 'k':
            case CSR_UP:
                cursor_pos -= cursor_up(cursor_pos,
                    config_table[CT_BEEP].ct_val);
                break;
            case 'u':
            case CSR_LFT:
                strcpy(cur_wd, "Reading new directory...");
                display_cwd();
                refresh_cwin();
                new_dir("..", curfilter);
                break;
            case ' ': /* space */
                if(menu_num == MENUDES) break;
                if(TST_FLG((entries + fl_first + cursor_pos)->e_flags, EMP_DIR))
                    break;
                if((xx = marked(fl_first + cursor_pos)) != -1)
                    remove_mark(xx);
                else
                    place_mark();
                cursor_pos += cursor_down(cursor_pos, NO_BEEP);
                break;
            default:
                if(num_marked)
                    sort_marked_list();
                rc = do_command(chr);
        }
        if(rc == RCQUIT)
            CONTINUE = 0;
        else
            display_fwin();
    }
    end_screen();
    system("clear");
    undo_stderr();
    return 0;
}

/*
 * exit_program() - Handle exiting from program when user presses intr
 *                  key.
 */
void
exit_program(void)
{
    end_screen();
    system("clear");
    undo_stderr();
    exit(1);
}

/*
 * tm2str() - converts values in tm struct into the string format used for
 *            screen display.  dd/mm/yy
 */
void
tm2str(struct tm *tt, char *tstr)
{
    sprintf(tstr, TMFORMAT, tt->tm_mday, tt->tm_mon + 1,
            tt->tm_year, tt->tm_hour, tt->tm_min);
}

/*
 * mod2str() - converts the mode bits in the stat struct into 2 strings
 *             which are used in the screen display.  The flags and the
 *             mode strings.
 */
void
mod2str(mode_t m, char *mstr)
{
    char type;

    type = " pc d b - l s   "[(m & S_IFMT) >> 12];
    sprintf(mstr, "%c%s%c%s%c%s%c",
            type,
            rwperms[(m & 0600) >> 7],
            (m & S_IEXEC) ?
                (m & S_ISUID) ? 's' : 'x' : (m & S_ISUID) ? 'S' : '-',
            rwperms[(m & 060) >> 4],
            (m & 010) ? (m & S_ISGID) ? 's' : 'x' : (m & S_ISGID) ? 'S' : '-',
            rwperms[(m & 006) >> 1],
            (m & 1) ? (m & 01000) ? 't' : 'x' : (m & 01000) ? 'T' : '-');
}

/*
 * fl_str() - Fill up a file list string for display.
 */
void
fl_str(struct entry *ep)
{
    struct tm *tt;
    char tstr[30], mstr[30], nstr[MAXPATHLEN];

    bzero(nstr, MAXPATHLEN);
#ifdef MALLOCDEBUG
    strcpy(tstr, "dd/mm/yy hh:mm");
#else
    tt = localtime(&(ep->e_stat.st_mtime));
    tm2str(tt, tstr);
#endif
    mod2str(ep->e_stat.st_mode, mstr);
    fixname(ep->e_name, nstr);
    sprintf(ep->e_flstr, flfmtstr, nstr, mstr, ep->e_stat.st_size,
            tstr);
}

/*
 * fixname() - Fix a file name to fit in the available space in the file
 *             list window.
 */
void
fixname(char *orig, char *new)
{
    if(strlen(orig) > maxfname) {
        strncpy(new, orig, maxfname - 3);
        strcat(new, " >>");
    } else {
        strcpy(new, orig);
    }
}

/*
 * set_fl_indices() - Set the fl_first and fl_last and cursor_pos values
 *                    to appropriate values based on a new fl_first.
 */
void
set_fl_indices(int newfl_first, int warp)
{
    int x;

    if(newfl_first < 0)
        fl_first = 0;
    else if(newfl_first >= count)
        fl_first = count - 1;
    else
        fl_first = newfl_first;
    fl_last = (((count - 1) - fl_first) <  fwinlines) ?
        count - 1 : fl_first + fwinlines - 1;
    x = fl_last - fl_first;
    if(warp == NOWARP)
        cursor_pos = (x < cursor_pos) ? x : cursor_pos;
    else
        cursor_pos = (warp == TOTOP) ? 0 : x;
}

/* 
 * Alloc() - General heap allocation.  Retries malloc if it fails.
 *           Written by: Rob Good <rcgood@fpg.uwaterloo.ca>
 */
void *
Alloc(int size)
{
    char *tmp;
    int tries=MEMTRIES;

    /*
     * Just return if the size is 0
     */
    if(!size) return((void *)0);

    /*
     *  Be persistant at trying to malloc() memory
     *  just in case it's a temporary condition.
     */
    while(!(tmp = (char *)malloc((unsigned)size))) {
        if(tries--) {
            sleep(2);
            continue;
        }
        end_screen();
        fprintf(stderr,
                "%s: Alloc() tried %d time%s and failed; pid: %d",
                progname, MEMTRIES, MEMTRIES==1?"":"s", getpid()
        );
        exit(1);
    }
    return((void *)tmp);
}

/*
 * RealFree() - Along the same lines as Alloc(), free() an object, then set
 *              the pointer equal to (NULL).
 */
void *
RealFree(void *to_free)
{
    if(to_free)
        free(to_free);
    return((void *)0);
}

/*
 * my_system() - pass a command to system() and then perhaps wait for the
 *               user to press a key.
 */
void
my_system(char *cmd, int pause)
{
    char c;
    int status;

    signal(SIGWINCH, SIG_DFL);
    signal(SIGINT, SIG_DFL);
    signal(SIGQUIT, SIG_DFL);
    puts(cmd);
    setstderr();
    status = system(cmd);
    seterrfile();
    if(pause) {
        if(status)
            printf("Command terminated with exit code: %d\n", status>>8);
        printf("\n%s", ENTRET_PROMPT);
        do {
            c = getchar();
        } while(c != '\r' && c != '\n');
    }
    signal(SIGWINCH, window_change_size);
    signal(SIGINT, exit_program);
    signal(SIGQUIT, exit_program);
}

/*
 * check_recursive_cp() - Try and figure out if the user is attempting
 *                        (perhaps unknowingly) to 'cp -r' a directory
 *                        into itself or one of its children.
 */
int
check_recursive_cp(char *a, char *b)
{
    char ta[MAXPATHLEN], tb[MAXPATHLEN];

    realpath(a, ta);
    realpath(b, tb);
    return(!strncmp(ta, tb, strlen(ta)));
}

/*
 * setup_stderr() - redirect stderr to a file in /tmp so that we don't
 *                  get it messing up our nice curses screen :)
 */
void
setup_stderr(void)
{
    char tmpf[40];

    sprintf(tmpf, "/tmp/FiLeS%dtmp", getpid());
    if((tmpstderrfd = dup(2)) == -1) {
        print_error("FATAL ERROR: Could not dup() stderr.");
        exit_program();
    }
    if((errfile = fopen(tmpf, "w+")) != NULL) {
        unlink(tmpf);
    } else {
        print_error("FATAL ERROR: Could not open error file.");
        exit_program();
    }
    seterrfile();
}

/*
 * undo_stderr() - undo what setup_stderr() has done.
 */
void
undo_stderr(void)
{
    fclose(errfile);
    close(tmpstderrfd);
}

/*
 * pass_signal() - pass a received signal onto a child process
 */
void
pass_signal(int s)
{
    kill(cpid, s);
}

/*
 * Run a command in the small syswin window
 */
void
syswin_system(char *bin, char *swch, char *f2)
{
    int status, i = 0, j, chk_tmp = FALSE, expan;
    char *cmd[5], tmp[MAXPATHLEN];
    char p[MAXPATHLEN];

    cmd[i++] = bin;
    if(swch)
        cmd[i++] = swch;

    init_syswin();

    for(j = 0; j < num_marked; j++) {

        status = 0;
        cmd[i++] = (entries + *(marked_list + j))->e_name;

        if(f2) {
            strcpy(tmp, f2);
            expan = expand_cmd(tmp, (entries + *(marked_list + j)));
            cmd[i++] = tmp;
            chk_tmp = TRUE;
        }
        cmd[i] = 0;

        if(S_ISDIR((entries + *(marked_list + j))->e_stat.st_mode)) {
            if(check_recursive_cp(cmd[i-2], cmd[i-1])) {
                sprintf(p, "Recursive copy: %s is a subdir of %s. %s",
                    cmd[i-1], cmd[i-2], "Continue? (y/n) "
                );
                if(!ask_user(p, 'n'))
                    goto __sw_cont;
            }
        }

        /*
         * Attempt to figure out if a [r]ename will cause us to
         * clobber our files by moving multiple files to a single
         * file destination.
         * Logic: If expand_cmd() made no expansions on the destination,
         *        and we are doing a [r]ename, and there are multiple
         *        files marked then test the destination to see if it
         *        is a regular file, or it doesn't exist.
         */
        if(!expan && !strcmp(PATH_MV, bin) && num_marked > 1) {
            struct stat tmpst;

            if(stat(tmp, &tmpst) || S_ISREG(tmpst.st_mode)) {
                if(S_ISREG((entries + *(marked_list + j))->e_stat.st_mode)) {
                    sprintf(p, "Multiple files selected and the %s ",
                               "destination is a file.  Are you sure? (y/n)");
                    if(!ask_user(p, 'n'))
                        goto __leave_syswin_system;
                }
            }
        }

        sprintf(p, "%s %s %s %s\n", cmd[0] ? cmd[0] : "", cmd[1] ? cmd[1] : "",
                cmd[2] ? cmd[2] : "", cmd[3] ? cmd[3] : ""
        );
        syswin_addstr(p);

        rewind(stderr);
        if(chk_tmp) {
            if(!tmp[0]) {
                status = 1;
                fprintf(stderr, "Command line too long\n");
                goto __sw_cont;
            }
        }

        cpid = fork();
        switch(cpid) {
            case -1:
                print_error("Could not fork() child process.");
                goto __leave_syswin_system;
                break;
            case 0:
                signal(SIGINT, SIG_DFL);
                signal(SIGQUIT, SIG_DFL);
                execv(cmd[0], cmd);
                _exit(1);
                break;
            default:
                signal(SIGINT, pass_signal);
                signal(SIGQUIT, pass_signal);
                while(!wait(&status) && errno == EINTR) {
                        kill(cpid, SIGINT);
                }
        }

__sw_cont:

        if(status) {
            fflush(stderr);
            get_error_str(status);
            if(j == num_marked - 1)
                syswin_wait();
        }
        i -= 2;
    }

__leave_syswin_system:

    signal(SIGINT, exit_program);
    signal(SIGQUIT, exit_program);

    end_syswin();
    touchwin(cwin);
    init_cwin();
    display_fwin();
    refresh_screen();
}

/*
 * get_error_str() - get the error string from the stderr file.
 */
void
get_error_str(int status)
{
    char emsg[MAXPATHLEN];

    emsg[0] = '\0';
    rewind(errfile);
    fgets(emsg, MAXPATHLEN, errfile);
    if(!emsg[0])
        sprintf(emsg,
                "Command terminated with error code %d.\n",
                status);
    syswin_msg(emsg);
}

/*
 * print_error() - print an error message to the user.
 */
void
print_error(char *basemsg)
{
    char mess[MAXPATHLEN];

    sprintf(mess, "%s %s", basemsg, sys_errlist[errno]);
    warn_user(mess);
}

/*
 * marked() - Is the file 'which' in the marked list?
 */
int
marked(int which)
{
    int i, FND = -1;

    for(i = 0; i < num_marked && FND == -1; i++) {
        if(*(marked_list + i) == which)
            FND = i;
    }
    return(FND);
}

/*
 * sort_marked_list() - Sort the marked list so that files are done in
 *                      order.
 */
void
sort_marked_list(void)
{
    qsort(marked_list, num_marked, sizeof(int), intp_cmp);
}

int
intp_cmp(int *a, int *b)
{
    return((*a > *b) ? 1 : (*a == *b) ? 0 : -1);
}

/*
 * handle_sigsegv() - SEGV signal handler function
 */
void
handle_sigsegv(void)
{
#ifdef DEBUG
    chdir(HOME_DDIR);
#endif
    end_screen();
    puts("ACK! Segmentation fault!");
    puts("Something went wrong, and I can't fix it. Sorry.");
    abort();
}

/*
 * basename() - grap only the last element of a path
 */
char *
basename(char *fullname)
{
    char *tmp;

    tmp = rindex(fullname, '/');
    if(!tmp)
        return(fullname);
    else
        return(++tmp);
}
