/*
 * $Source: /u/pfratar/src/files-2.1/src/RCS/sorting.c,v $
 *
 * $Id: sorting.c,v 6.7 1993/03/30 22:12:20 pfratar v2_1pl10 $
 *
 * files - full screen file manager for UNIX
 * Copyright (C) 1993 Paul Frattaroli
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
#define EXTERN
#include "global.h"
#undef EXTERN

#ifndef lint
static char *rcsid = "$Id: sorting.c,v 6.7 1993/03/30 22:12:20 pfratar v2_1pl10 $";
#endif

/*
 * fl_extn_sort() - Used by sort_display() to sort on extension.
 */
int
fl_extn_sort(struct entry *e1, struct entry *e2)
{
    char *p, *q;

    p = index(e1->e_name, '.');
    q = index(e2->e_name, '.');
    if(!p && !q) {
        return(strcmp(e1->e_name, e2->e_name));
    } else if(!p && q) {
        return(-1);
    } else if(p && !q) {
        return(1);
    } else {
        p++;
        q++;
        return(strcmp(p, q));
    }
}

/*
 * fl_time_sort(), time_t_cmp() - Used by sort_display() to sort on time.
 */
int
fl_time_sort(struct entry *e1, struct entry *e2)
{
    return(time_t_cmp(e1->e_stat.st_mtime, e2->e_stat.st_mtime));
}

int
time_t_cmp(time_t a, time_t b)
{
    return((a > b) ? -1 : (a == b) ? 0 : 1);
}

/*
 * fl_size_sort(), off_t_cmp() - Used by sort_display() to sort on size.
 */
int
fl_size_sort(struct entry *e1, struct entry *e2)
{
    return(off_t_cmp(e1->e_stat.st_size, e2->e_stat.st_size));
}

int
off_t_cmp(off_t a, off_t b)
{
    return((a > b) ? -1 : (a == b) ? 0 : 1);
}

/*
 * fl_name_sort() - Used by sort_display() to sort on name.
 */
int
fl_name_sort(struct entry *e1, struct entry *e2)
{
    return(strcmp(e1->e_name, e2->e_name));
}

/*
 * fl_def_sort() - Used by sort_display() to sort in FILES's default
 *                 manner.
 */
int
fl_def_sort(struct entry *e1, struct entry *e2)
{
    int num_of_dirs = 0, x, y, rc;
    struct stat *tmpst;

    /*
     * Various cases:
     *    1)   . and ..
     *    2)   directory and file
     *    3)   2 directories
     *    4)   2 files
     */
    x = e1->e_stat.st_mode;
    y = e2->e_stat.st_mode;
    x &= 0170000;
    y &= 0170000;
    x >>= 12;
    y >>= 12;

    if(x == 012) {
        tmpst = (struct stat *)Alloc(sizeof(struct stat));
        stat(e1->e_name, tmpst);
        x = tmpst->st_mode;
        x &= 0170000;
        x >>= 12;
        Free(tmpst);
    }
    if(y == 012) {
        tmpst = (struct stat *)Alloc(sizeof(struct stat));
        stat(e2->e_name, tmpst);
        y = tmpst->st_mode;
        y &= 0170000;
        y >>= 12;
        Free(tmpst);
    }

    /*
     *                                   num_of_dirs
     * neither e1 or e2 are directories:      0
     * e1 is a directory:                     1
     * e2 is a directory:                     2
     * both e1 and e2 are directories:        3
     */
    if(x == 004) num_of_dirs |= 0x01;
    if(y == 004) num_of_dirs |= 0x02;

    switch(num_of_dirs) {
        case 0:
            return(fl_name_sort(e1, e2));
            break;
        case 1:
            return(-1);
            break;
        case 2:
            return(1);
            break;
        case 3:
            if(!strcmp(e1->e_name, ".")) {
                rc = -1;
            } else if(!strcmp(e2->e_name, ".")) {
                rc = 1;
            } else if(!strcmp(e1->e_name, "..")) {
                rc = -1;
            } else if(!strcmp(e2->e_name, "..")) {
                rc = 1;
            } else {
                rc = fl_name_sort(e1, e2);
            }
            break;
    }
    return(rc);
}

/*
 * sort_display() - Sort the file list display in a different order.
 */
int
sort_display(void)
{
    int stype;

    stype =
        get_char_ans("Sort: [d]efault, [e]xtension, [n]ame, [t]ime, [s]ize? ");
    switch(stype) {
        case 'e':
            sort_type = SBYEXTN;
            break;
        case 't':
            sort_type = SBYTIME;
            break;
        case 's':
            sort_type = SBYSIZE;
            break;
        case 'n':
            sort_type = SBYNAME;
            break;
        default:
            sort_type = SBYDFLT;
    }
    num_marked = 0;
    new_dir(".", curfilter);
    return(RCNOERR);
}

